#!/bin/sh

set -e
set -x

# Qmail is originally from http://pobox.com/~djb/software
# The TLS patch is from # http://www.esat.kuleuven.ac.be/~vermeule/qmail
FILE=qmail-1.03

cd /usr/local/src
tar xzf /dist/$FILE.tar.gz

cd $FILE

mkdir -p /var/qmail
addqmuser() { if [ "$4" ]; then group="-G $4"; else group=""; fi
              id $1 >/dev/null 2>&1 || 
              /usr/sbin/adduser -g $2 -s /bin/true -d $3 $group $1 }
addqmuser alias nobody /var/qmail/alias
addqmuser qmaild nobody /var/qmail
addqmuser qmaill nobody /var/qmail
addqmuser qmailp nobody /var/qmail
addqmuser qmailq mail /var/qmail
addqmuser qmailr mail /var/qmail
addqmuser qmails mail /var/qmail

# Change qmail group conf to work with cyrus
cat <<'...EOF...' >conf-groups
mail
nobody
...EOF...

patch -sp1 </usr/local/src/qmail-patches/qmail-dpipe-socket.patch
patch -sp1 </dist/qmail-tls.patch

make setup check
# This fixes stupid hardcoded path in Makefile
mkdir -p /usr/local/ssl/bin
(cd /usr/local/ssl/bin ; ln -s ../../../bin/openssl)
make cert || chown qmaild.mail /var/qmail/control/cert.pem
./config
(cd ~alias; touch .qmail-postmaster .qmail-mailer-daemon .qmail-root)
chmod 644 ~alias/.qmail*

cp /var/qmail/boot/home /var/qmail/rc
[ -e /var/qmail/users/assign ] || echo . >/var/qmail/users/assign

# Toss in a startup file
cd /etc/rc.d/init.d
cat >qmail <<'...EOF...'
. /etc/rc.d/init.d/functions

[ -f /var/qmail/rc ] || exit 0

case "$1" in
  start)
    echo -n "Starting Qmail: "
    /var/qmail/rc &
    echo $! >/var/lock/subsys/qmail
    echo_success
    echo
    ;;
  stop)
    echo -n "Stopping Qmail: "
    kill `cat /var/lock/subsys/qmail`
    rm -f /var/lock/subsys/qmail
    echo_success
    echo
    ;;
  status)
    status qmail-send
    ;;
  restart|reload)
    $0 stop
    sleep 1
    $0 start
    ;;
  *)
    echo "Usage: qmail {start|stop|status|restart|reload}"
    exit 1
esac

exit 0
...EOF...

chmod 755 qmail

cd ..
for i in 0 1 2 6; do
  cd rc$i.d
  ln -vsf ../init.d/qmail K52qmail
  cd ..
done

for i in 3 4 5; do
  cd rc$i.d
  ln -vsf ../init.d/qmail S52qmail
  cd ..
done

# Let the dpipe take care of user existence
echo :/var/dpipe/socket >~alias/.qmail-default
