#include "conf.h"
#include <libdbd.h>

#include <stdio.h>
#include <ctype.h>

char *dpipe_opt_domain;
char *dpipe_opt_inbox;
char *dpipe_opt_inject;

struct module_conf *read_conf(char *file) {
  dbd_conf_t *conf;
  dbd_conf_section_t *s;
  struct module_conf *mods = NULL, *tail = NULL, *mod;
  int when;
  const char *opt;
  
  if ((conf = dbd_conf_parse_file(file)) == NULL)
    return NULL;

  for (s = conf->sections; s; s = s->next) {
    if (strcasecmp(s->name, "before") == 0) when = MODULE_WHEN_BEFORE;
    else if (strcasecmp(s->name, "after") == 0) when = MODULE_WHEN_AFTER;
    else when = -1;
      
    if (when != -1) {
      dbd_conf_element_t *v;

      for (v = s->elements; v; v = v->next) {
        if (strcasecmp(v->key, "module") == 0) {
          bzero(mod = dbd_malloc (sizeof (*mod)), sizeof (*mod));

          for (mod->conf = conf->sections; mod->conf; mod->conf = mod->conf->next) {
            char *parse[2], *dup;

            dup = dbd_strdup(mod->conf->name);
            if ((dbd_split(" ", dup, 2, parse, 0) == 2) &&
                (strcasecmp(parse[0], "module") == 0) &&
                (strcasecmp(parse[1], v->value) == 0)) {
              free(dup);
              break;
            }
            free(dup);
          }
          
          if (!mod->conf) {
            dbd_log(LOG_ERR, "%s: Can't find configuration section for module %s", file, v->value);
            goto error;
          }
          
          mod->when = when;
          mod->name = dbd_strdup(v->value);

          if (tail) {
            tail->next = mod;
            tail = mod;
          } else {
            mods = tail = mod;
          }
        } else {
          dbd_log(LOG_ERR, "%s: Unknown before/after option: %s", file, v->key);
          goto error;
        }
      }
    }
  }

  if ((opt = dbd_conf_find(conf, "global", "inject")) != NULL) {
    dpipe_opt_inject = dbd_strdup(opt);
  } else {
    dpipe_opt_inject = dbd_strdup("/var/qmail/bin/inject");
  }    

  if ((opt = dbd_conf_find(conf, "global", "inbox")) != NULL) {
    dpipe_opt_inbox = dbd_strdup(opt);
  } else {
    dpipe_opt_inbox = dbd_strdup("INBOX");
  }    

  if ((opt = dbd_conf_find(conf, "global", "domain")) != NULL) {
    dpipe_opt_domain = dbd_strdup(opt);
  } else {
    if (gethostname(dpipe_opt_domain = dbd_malloc(1024), 1024) == -1) {
      free(dpipe_opt_domain);
      dpipe_opt_domain = NULL;
    }
  }    

  if ((opt = dbd_conf_find(conf, "global", "deliver")) != NULL)
    deliver_set_path(opt);
  else
    deliver_set_path("/usr/cyrus/bin/deliver $u");
  
  if ((opt = dbd_conf_find(conf, "global", "deliver to folder")) != NULL)
    deliver_set_path_to_folder(opt);
  else
    deliver_set_path_to_folder("/usr/cyrus/bin/deliver -a $u -m $s $u");
  
  /* don't destroy conf, sections are still used by module_conf */

  return mods;

  error:

  dbd_conf_destroy(conf);
  for (mod = mods; mod; mod = mod->next) {
    if (mod->name) free(mod->name);
    free(mod);
  }
  return NULL;
}

dbd_conf_element_t *conf_find_key(dbd_conf_section_t *s,
                                  dbd_conf_element_t *v,
                                  const char *key) {
  if (!v) v = s->elements;
  else v = v->next;
  
  while (v) {
    if (strcasecmp(v->key, key) == 0)
      return v;
    v = v->next;
  }
  
  return NULL;
}
