#include <stdlib.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <sys/uio.h>
#include <sysexits.h>
#include <syslog.h>

#include "daemondefs.h"

#define MYNAME "dpipe-deliver"

#define BARF(lvl, ex, err...) {                                    \
	  openlog(MYNAME, LOG_PERROR, LOG_DAEMON);                       \
		syslog(lvl, err);                                              \
		exit(ex);                                                      \
	}

int main(int argc, char *argv[]) {
  int s, n, i;
	struct sockaddr_un sa = { AF_UNIX, UNSOCKET_LOCATION };
	struct iovec header[] = {
		{ "LOCAL ", 6 },
		{ NULL, 0 },
		{ "\n\n", 2 }
	};
	int nheader = sizeof (header) / sizeof (struct iovec);
	char buf[4096];

  if (argc != 2)
		BARF(LOG_ERR, EX_USAGE, "usage: %s <local recipient>", MYNAME);

	header[1].iov_base = argv[1];
	header[1].iov_len = strlen(argv[1]);

	if ((s = socket(PF_UNIX, SOCK_STREAM, 0)) == -1)
		BARF(LOG_ERR, EX_OSERR, "socket(PF_UNIX, SOCK_STREAM, 0) failed: %m");

	if (connect(s, (struct sockaddr *)&sa, sizeof (sa)) == -1)
	  BARF(LOG_ERR, EX_CONFIG, "connect(%s): %m", UNSOCKET_LOCATION);

	for (n = 0, i = 0; i < nheader; i++)
		n += header[i].iov_len;

  if ((i = writev(s, header, nheader)) != n) {
		if (i == -1)
			BARF(LOG_ERR, EX_IOERR, "error writing to dpipe socket %s: %m", UNSOCKET_LOCATION)
		else
			BARF(LOG_ERR, EX_IOERR, "short write to dpipe socket %s", UNSOCKET_LOCATION)
	}

	while ((n = read(0, buf, sizeof (buf))) != 0) {
		if (n == -1)
		  BARF(LOG_ERR, EX_IOERR, "error reading message: %m");

		if ((i = write(s, buf, n)) != n) {
			if (i == -1)
				BARF(LOG_ERR, EX_IOERR, "error writing to dpipe socket %s: %m", UNSOCKET_LOCATION)
			else
				BARF(LOG_ERR, EX_IOERR, "short write to dpipe socket %s", UNSOCKET_LOCATION)
		}
	}

	if (shutdown(s, SHUT_WR) == -1)
		BARF(LOG_ERR, EX_OSERR, "can't shutdown socket %s for writing: %m", UNSOCKET_LOCATION);

	if ((n = read(s, buf, 4)) == -1)
		BARF(LOG_ERR, EX_IOERR, "error reading dpipe reply: %m");

  buf[n-1] = '\0';

	switch (i = atoi(buf)) {
		case 0   :
			exit(EX_OK);
			break;
		case 100 :
		  exit(EX_NOUSER);
			break;
		case 111 :
		  exit(EX_TEMPFAIL);
			break;
		default :
			exit(i);
			break;
	}

	return 1;
}

