#include "module_dl.h"
#include "dpipe_mod.h"
#include <libdbd.h>

#include <dlfcn.h>

#ifndef DAEMONIZE
int process_dl_module(struct module_conf *mod, struct message *msg) {
  void *dh;
  int (*process_message)(struct message *msg, void *opts);
  void *(*configure_module)(dbd_conf_section_t *conf);
  void *opts = NULL;
  int ret;
  char *module_path;
  dbd_conf_element_t *v;
  
  if ((v = conf_find_key(mod->conf, NULL, "path")) == NULL) {
    dbd_log(LOG_ERR, "%s: no path found\n", mod->name);
    return MESSAGE_STATUS_TEMPORARY_ERROR;
  }

  if (*v->value == '/')
    module_path = dbd_strdup(v->value);
  else {
    module_path =
      dbd_malloc((sizeof (MODULE_DIR) - 1) + 1 + strlen(v->value) + 1);
    sprintf(module_path, "%s/%s", MODULE_DIR, v->value);
  }
  
  dh = dlopen(module_path, RTLD_NOW);
  free(module_path);
        
  if(!dh) {
    dbd_log(LOG_ERR, "%s", dlerror());
    return MESSAGE_STATUS_TEMPORARY_ERROR;
  }

  configure_module = dlsym(dh, MODULE_DL_CONF_FUNC);
  process_message = dlsym(dh, MODULE_DL_PROC_FUNC);
  
  if(!process_message) {
    dbd_log(LOG_ERR, "%s", dlerror());
    return MESSAGE_STATUS_TEMPORARY_ERROR;
  }

  dbd_opt_set_log_identity(mod->name);
  if (configure_module)
    opts = configure_module(mod->conf);
  
  ret = process_message(msg, opts);
  dbd_opt_set_log_identity("dpipe");
  
  dlclose(dh);
  return ret;
}
#else
int preload_dl_module(struct module_conf *mod) {
  void *dh;
  char *module_path;
  void *(*configure_module)(dbd_conf_section_t *conf);
  dbd_conf_element_t *v;
  
  if ((v = conf_find_key(mod->conf, NULL, "path")) == NULL) {
    dbd_log(LOG_ERR, "%s: no path found\n", mod->name);
    return -1;
  }

  if (*v->value == '/')
    module_path = dbd_strdup(v->value);
  else {
    module_path =
      dbd_malloc((sizeof (MODULE_DIR) - 1) + 1 + strlen(v->value) + 1);
    sprintf(module_path, "%s/%s", MODULE_DIR, v->value);
  }
  
  dh = dlopen(module_path, RTLD_NOW);
  free(module_path);
        
  if(!dh) {
    dbd_log(LOG_ERR, "%s", dlerror());
    return -1;
  }

  configure_module = dlsym(dh, MODULE_DL_CONF_FUNC);
  mod->process_message = dlsym(dh, MODULE_DL_PROC_FUNC);
  
  if(!mod->process_message) {
    dbd_log(LOG_ERR, "%s", dlerror());
    return -1;
  }

  dbd_opt_set_log_identity(mod->name);
  if (configure_module)
    mod->options = configure_module(mod->conf);
  dbd_opt_set_log_identity("dpipe");
  
  return 0;
}
#endif

