/* Various functionality for dropping root privileges. */
#include "libdbd.h"
#include "privs.h"

#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
#include <stdio.h>

static int get_ids(uid_t *uid, gid_t *gid) {
  struct passwd *pwd;
  struct group *grp;

  pwd = getpwnam(DPIPE_USER);
  if(!pwd) {
    dbd_error(LOG_ERR, "Unable to get UID of %s", DPIPE_USER);
    return -1;
  }
  *uid = pwd->pw_uid;
  
  grp = getgrnam(DPIPE_GROUP);
  if(!grp) {
    dbd_error(LOG_ERR, "Unable to get GID of %s", DPIPE_GROUP);
    return -1;
  }
  *gid = grp->gr_gid;

  return 0;
}
  
int drop_privs() {
  uid_t uid; /* we will run as this uid */
  gid_t gid; /* we will run as this gid */

  if(get_ids(&uid, &gid) < 0) {
    dbd_log(LOG_ERR, "Unable to drop privs.");
    return -1;
  }

  if(initgroups(DPIPE_USER, gid) < 0) {
    dbd_error(LOG_ERR, "Unable to initgroups for %s", DPIPE_USER);
    return -1;
  }

  if(setgid(gid) < 0) {
    dbd_error(LOG_ERR, "Unable to setgid to %s (%u)", DPIPE_GROUP, 
        (unsigned)gid);
    return -1;
  }

  if(setuid(uid) < 0) {
    dbd_error(LOG_ERR, "Unable to setuid to %s (%u)", DPIPE_USER,
        (unsigned)uid);
    return -1;
  }

  return 0;
}
