#ifndef DAEMON_DECRYPT_H
#define DAEMON_DECRYPT_H

#define DAEMON_DINFO_SIGNATURE 0x1
#define DAEMON_DINFO_DECRYPTED 0x2
#define DAEMON_DINFO_PLAINTEXT 0x3

extern char *daemon_detached_data;
extern unsigned long daemon_detached_data_len;

void daemon_detached_data_set(char *s, unsigned long len);
void daemon_detached_data_free(void);

struct daemon_dinfo {
    int type; /* sig or decrypted */
    int checked; /* have we already recursed through this plaintext? */
    union {
        struct {
            u32 keyid1; 
            u32 keyid2;
            char *name;
            u32 timestamp;
            int status;
            struct daemon_dinfo *parent;
        } signature;
        struct {
            int status;
            unsigned long crypt_len;
            char *crypt_buf;
        } decrypted;
        IOBUF out;
    } data;
    struct daemon_dinfo *next;
    struct daemon_dinfo *children;
};

struct daemon_notable {
  unsigned long begin;
  unsigned long end;
  int type;
  u32 keyid1;
  u32 keyid2;
  u32 timestamp;
  char *name;
  int status;
};

struct daemon_dbuf {
  char *buf;
  unsigned long len;
  struct daemon_notable *notables;
  int nnotables;
};

struct daemon_dinfo *daemon_dinfo_new(int type);
void daemon_finish_signature(struct daemon_dinfo *info, 
        struct daemon_dinfo *chunk, u32 keyid1, u32 keyid2, char *name, 
        unsigned long namelen, u32 timestamp, int status);
void daemon_finish_decrypted(struct daemon_dinfo *info, 
        struct daemon_dinfo *chunk, int status);
void daemon_finish_plaintext(struct daemon_dinfo *info,
        struct daemon_dinfo *chunk);
void daemon_dinfo_free(struct daemon_dinfo *p);
void daemon_dbuf_free(struct daemon_dbuf *p);

int daemon_decrypt(const char *src, int srcsize, struct daemon_dbuf **dbuf, 
		int signature, int encryption);

#endif /* DAEMON_DECRYPT_H */
