#include <config.h>
#include <string.h>
#include <syslog.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include "util.h"
#include "options.h"

/* How many directories of depth to hash. */
#define HASH_DEPTH 2
/* How many "buckets" or directories in each directory. */
#define HASH_BUCKETS 50
/* The maximum strlen of a bucket when sprintf'd. Used to calculate maximum
   string length of the directory. */
#define BUCKET_STRLEN 2

char *daemon_get_homedir(const char *user) {
  int i;
  int vals[HASH_DEPTH];
  char *dir;
  char buf[BUCKET_STRLEN+2];
  
  memset(vals, 0, HASH_DEPTH * sizeof(int));
  
  for(i = strlen(user) - 1; i >= 0; i--)
    vals[i%HASH_DEPTH] += user[i];
  
  /* We need enough for the key root, each bucket length followed by
     a slash, followed by the username and a trailing NULL. */
  dir = m_alloc(HASH_DEPTH * (BUCKET_STRLEN + 1) + strlen(user) + 1);
  dir[0] = '\0';
  for(i = 0; i < HASH_DEPTH; i++) {
    snprintf(buf, BUCKET_STRLEN+1, "%d", vals[i] % HASH_BUCKETS);
    strcat(dir, buf);
    strcat(dir, "/");
  }
  strcat(dir, user);

  return dir;
}
	

