#include "daemon_keyserver.h"

#include "config.h"
#include "util.h"
#include "http.h"

#include <stdio.h>
#include <string.h>

static char *initial_keyserver = NULL;
#define KEYSERVER_SIZE 256 /* name of keyserver */
#define URL_SIZE 128 /* URL size without keyserver name */
#define REQUEST_SIZE (KEYSERVER_SIZE+URL_SIZE) 

void daemon_keyserver_set_initial(const char *s) {
    initial_keyserver = m_strdup(s);
}

static char last_keyserver[KEYSERVER_SIZE];

void daemon_keyserver_restart() {
    last_keyserver[0] = '\0';
}

char *daemon_keyserver_get_next() {
    /* !! wishlist: keep track of past servers and do loop detection */
    
    if(*last_keyserver == '\0')
	return strncpy(last_keyserver, initial_keyserver, KEYSERVER_SIZE);
    else {
	struct http_context http;
	char request[REQUEST_SIZE]; 
	int rc;
	
	snprintf(request, REQUEST_SIZE, "http://%s:11371/pks/parent", 
		last_keyserver);

	rc = http_open_document(&http, request, 0);
	if(rc || http.status_code != 200)
	    last_keyserver[0] = '\0';
	else {
	    unsigned char *buf = NULL;
	    int len = 0;
	    unsigned result;
	    unsigned max_length = KEYSERVER_SIZE;

	    result = iobuf_read_line(http.fp_read, &buf, &len, &max_length);
	    http_close(&http);

	    if(!result)
		last_keyserver[0] = '\0';
	    else {
		result--; /* chomp newline */
		if(result >= KEYSERVER_SIZE)
		    last_keyserver[0] = '\0';
		else {
		    memmove(last_keyserver, buf, result);
		    last_keyserver[result] = '\0';
		}
	    }
	}
	if(last_keyserver[0])
	    return last_keyserver;
	else
	    return NULL;
    }
    /* not reached */
}

