#ifndef FETCH_H
#define FETCH_H

#include "imap_seq.h"


typedef enum {
  IMAP_FETCH_BODY_WHOLE,
  IMAP_FETCH_BODY_TEXT,
  IMAP_FETCH_BODY_HEADER,
  IMAP_FETCH_BODY_HEADER_FIELDS,
  IMAP_FETCH_BODY_HEADER_FIELDS_NOT,
  IMAP_FETCH_BODY_MIME
} fetch_body_request_type;

/* BODY - .PEEK? - [ - part.? - type? - ] - <partial_start.partial_end>? */
typedef struct _fetch_body_request {
  int peek;

  char *part;

  fetch_body_request_type type;

  char **headers;
  int nheaders;

  unsigned int partial_start, partial_len;

  struct _fetch_body_request *next;
} fetch_body_request;


typedef struct _fetch_request {
  char *tag, *ptag;

  imap_sequence *seq;

  unsigned int by_uid  : 1,

    want_body          : 1,
    want_bodies        : 1,
    want_bodystructure : 1,
    want_envelope      : 1,
    want_flags         : 1,
    want_internaldate  : 1,
    want_rfc822        : 1,
    want_rfc822_size   : 1,
    want_rfc822_header : 1,
    want_rfc822_text   : 1,
    want_uid           : 1;

  fetch_body_request *bodies;

  struct _fetch_request *next;
} fetch_request;

int cmd_STORE(imap_session *sess, char *tag, char *cmd);
int cmd_FETCH(imap_session *sess, char *tag, char *cmd);
int resp_FETCH(imap_session *sess, int msgno, char *response, char *text);

void fetch_request_destroy(fetch_request *f);
void fetch_body_request_destroy(fetch_body_request *fb);


typedef struct _fetch_response fetch_response;
struct _fetch_response {
  unsigned int has_envelope : 1,
    has_internaldate        : 1,
    has_uid                 : 1,
    has_flags               : 1,
    has_rfc822_header       : 1,
    has_body                : 1,
    has_body_header         : 1;

  char *internaldate;
  char *flags;
  char *envelope;
  
  unsigned int msgno, uid;

  char *body;
  unsigned int body_len;

  char *body_header;
  unsigned int body_header_len;
};

void fetch_resp_destroy(fetch_response *f);

#endif
