#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>

#include <libdbd.h>
#include <libmime.h>

#include "imapproxy.h"
#include "folder.h"
#include "imap_cmd.h"
#include "imap_resp.h"
#include "uidmap.h"

static int cb_new_folder(imap_session *sess, char *tag, char *text,
                         char *folder);

int cmd_SELECT(imap_session *sess, char *tag, char *cmd) {
  char *folder, *p;
  
  server_writecmd(sess, tag, cmd);

  if (strncasecmp(cmd, "SELECT ", 7) == 0) cmd += 7;
  else if (strncasecmp(cmd, "EXAMINE ", 8) == 0) cmd += 8;
  else {
    dbd_log(LOG_ERR, "WARNING - cmd_SELECT called for command: %s", cmd);
    return 1;
  }

  if (*cmd == '"') {
    cmd++;
    if ((p = strchr(cmd, '"')) == NULL)
      p = cmd + strlen(cmd);
    folder = dbd_strndup(cmd, p - cmd);
  } else
    folder = dbd_strdup(cmd);

  resp_callback(sess, tag, cb_new_folder, folder);
  
  if (sess->folder) {
    uidmap_clear(sess);
    free(sess->folder);
    sess->folder = NULL;
    sess->state = IMAP_STATE_AUTHENTICATED;
    sess->exists = 0;
  }
  
  return 1;
}


int cmd_UNSELECT(imap_session *sess, char *tag, char *cmd) {
  server_writecmd(sess, tag, cmd);

  resp_callback(sess, tag, cb_new_folder, NULL);

  if (sess->folder) {
    uidmap_clear(sess);
    free(sess->folder);
    sess->folder = NULL;
    sess->state = IMAP_STATE_AUTHENTICATED;
    sess->exists = 0;
  }
  
  return 1;
}


static int cb_new_folder(imap_session *sess, char *tag, char *text,
                         char *folder) {
  client_writeresp(sess, tag, text);
  
  if (strncmp(text, "BAD", 3) == 0) { /* failed command */
    free(folder);
    sess->uidvalidity = 0;
    return 1;
  }

  if (strncmp(text, "OK", 2) != 0) { /* no folder, but still unselected */
    free(folder);
    return 1;
  }

  sess->folder = folder;
  sess->state = IMAP_STATE_SELECTED;

  fixup_context_set_folder(sess->fixup, folder);

  return 1;
}
