/* imap_cmd.c - command handlers for imap proxy module */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <libdbd.h>

#include "imapproxy.h"
#include "imap_cmd.h"

#include "append.h"
#include "fetch.h"
#include "folder.h"
#include "login.h"
#include "starttls.h"

/***********************************************************************
 *
 * command handlers - cmd_*
 *
 * All command handlers should behave as follows:
 * 
 * takes: pointer to imap_session structure
 *        tag for command
 *        command
 *
 * returns: 1 to continue session, 0 to close
 *
 ***********************************************************************/

/* ugly macros, but less error-prone to maintain table entries */
#define CMD_HANDLER_TABLE_ENTRY(a) { sizeof(#a) - 1, #a, cmd_##a }
#define CMD_HANDLER_TABLE_ENTRY_2(a, f) { sizeof(#a) - 1, #a, cmd_##f }

typedef struct {
  int cmdlen;
  char *cmd;
  int (*chandler)();
} imap_cmdhandler;


static int cmd_passthru(imap_session *sess, char *tag, char *cmd);


/* each IMAP command has a handler mapped to it */
static const imap_cmdhandler client_cmd_table[] = {
  CMD_HANDLER_TABLE_ENTRY(APPEND),
  CMD_HANDLER_TABLE_ENTRY_2(CLOSE, UNSELECT),
  CMD_HANDLER_TABLE_ENTRY_2(EXAMINE, SELECT),
  CMD_HANDLER_TABLE_ENTRY(FETCH),
  CMD_HANDLER_TABLE_ENTRY(LOGIN),
  CMD_HANDLER_TABLE_ENTRY(SELECT),
  CMD_HANDLER_TABLE_ENTRY(STARTTLS),
  CMD_HANDLER_TABLE_ENTRY(STORE),
  CMD_HANDLER_TABLE_ENTRY(UNSELECT),
  { 0, "",     cmd_passthru }
  /* passthru handler is applied to all non-matching commands */
};


static int cmd_passthru(imap_session *sess, char *tag, char *cmd) {
  server_writecmd(sess, tag, cmd);
  return(1);
}


void *client_get_command_handler(imap_session *sess, const char *cmd) {
  imap_cmdhandler *handlerp;

  for (handlerp = (imap_cmdhandler *)&client_cmd_table[0];
       handlerp->cmd != NULL; handlerp++) {
    if (!strncasecmp(cmd, handlerp->cmd, handlerp->cmdlen)) {
      return(handlerp->chandler);
    }
  }
  return(handlerp->chandler);
}


void client_handle_command(imap_session *sess) {
  int (*cmd_fp)();
  char *cmd, *tag;
  
  if ((tag = client_getline(sess)) == NULL) return;

  if ((cmd = strchr(tag, ' ')) == NULL) {
    server_writeline(sess, tag);
    return;
  }

  *cmd = '\0';
  cmd++;

  if (strncasecmp(cmd, "UID ", 4) == 0)
    cmd_fp = client_get_command_handler(sess, cmd+4);
  else
    cmd_fp = client_get_command_handler(sess, cmd);
  if (!cmd_fp(sess, tag, cmd)) {
        /* serious problem, must go away now */
    sess->state = IMAP_STATE_UNCONNECTED;
    return;
  }
}


