#include <stdlib.h>
#include <string.h>
#include <libdbd.h>
#include <ctype.h>

#include "imap_seq.h"

static inline int seqnumcmp(unsigned long a, unsigned long b);


imap_sequence *imap_sequence_new(void) {
  imap_sequence *seq;

  bzero(seq = dbd_malloc(sizeof (imap_sequence)), sizeof (imap_sequence));
  return seq;
}


void imap_sequence_destroy(imap_sequence *seq) {
  while (seq) {
    imap_sequence *next = seq->next;
    free(seq);
    seq = next;
  }
}


char *imap_sequence_new_from_string(char *cmd, imap_sequence **out) {
  imap_sequence *h = NULL, *t = NULL, *seq;
  
  while (1) {
    if (*cmd == '*') {
      seq = imap_sequence_new();
      seq->first = seq->last = 0;
      cmd++;
    } else if (isdigit(*cmd)) {
      seq = imap_sequence_new();
      seq->first = seq->last = atoi(cmd);
      while (isdigit(*cmd)) cmd++;
    } else {
      imap_sequence_destroy(h);
      return NULL;
    }
        
    if (*cmd == ':') {
      cmd++;
      if (*cmd == '*') {
        seq->last = 0;
        cmd++;
      } else if (isdigit(*cmd)) {
        seq->last = atoi(cmd);
        while (isdigit(*cmd)) cmd++;
      } else {
        imap_sequence_destroy(h);
        imap_sequence_destroy(seq);
        return NULL;
      }
    }

    if (seqnumcmp(seq->last, seq->first) < 0) {
      unsigned long switcharoo = seq->first;
      seq->first = seq->last;
      seq->last = switcharoo;
    }
    
    if (t)
      t->next = seq;
    else
      h = seq;
    t = seq;
    
    if (*cmd == ',') {
      cmd++;
      continue;
    } else if (isspace(*cmd))
      break;
  }

  *out = h;
  while (isspace(*cmd)) cmd++;
  return cmd;
}


static inline int seqnumcmp(unsigned long a, unsigned long b) {
  if ((a == 0) && (b == 0)) return 0;
  if (a == 0) return 1000;
  if (b == 0) return -1000;
  return a - b;
}


int imap_sequence_has_message(imap_sequence *seq, unsigned long message) {
  while (seq) {
    if ((seqnumcmp(seq->first, message) <= 0) &&
        (seqnumcmp(seq->last, message) >= 0))
      return 1;

    seq = seq->next;
  }
  
  return 0;
}


imap_sequence *imap_sequence_remove(imap_sequence *seq, imap_sequence *new) {
  imap_sequence **head = &seq, *s, **ref;

  while (new) {
    head = ref = &seq;
    while ((s = *ref)) { /* ((shush gcc)) */
      if ((seqnumcmp(new->first, s->first) <= 0) &&
          (seqnumcmp(new->last, s->last) >= 0)) {
        *ref = s->next;
        s->next = NULL;
        imap_sequence_destroy(s);
        continue;
      }

      if ((seqnumcmp(new->first, s->first) >= 0) &&
          (seqnumcmp(new->last, s->last) <= 0)) {
        imap_sequence *s2;
        
        s2 = imap_sequence_new();
        s2->first = new->last + 1;
        s2->last = s->last;
        s2->next = s->next;
        s->last = new->first - 1;
        s->next = s2;
      } else if ((seqnumcmp(new->first, s->first) >= 0) &&
                 (seqnumcmp(new->first, s->last) <= 0)) {
        s->last = new->first - 1;
      } else if ((seqnumcmp(new->last, s->first) >= 0) &&
                 (seqnumcmp(new->last, s->last) <= 0)) {
        s->first = new->last + 1;
      }

      if (s->first > s->last) {
        *ref = s->next;
        s->next = NULL;
        imap_sequence_destroy(s);
      } else
        ref = &s->next;
    }

    new = new->next;
  }

  return *head;
}


imap_sequence *imap_sequence_remove_one(imap_sequence *seq,
                                        unsigned long message) {
  imap_sequence one;

  one.first = one.last = message;
  one.next = NULL;

  return imap_sequence_remove(seq, &one);
}


imap_sequence *imap_sequence_add(imap_sequence *seq, imap_sequence *new) {
  imap_sequence *s;

  while (new) {
    for (s = seq; s; s = s->next) {
      if ((seqnumcmp(new->first, s->first) >= 0) &&
          (seqnumcmp(new->last, s->last) <= 0))
        break;

      if ((seqnumcmp(new->first, s->first) <= 0) &&
          (seqnumcmp(new->last, s->last) >= 0)) {
        s->first = new->first;
        s->last = new->last;
        break;
      }

      if ((seqnumcmp(new->first, s->first) >= 0) &&
          (seqnumcmp(new->first, s->last) <= 1)) {
        s->last = new->last;
        break;
      }

      if ((seqnumcmp(new->last, s->first) >= -1) &&
          (seqnumcmp(new->last, s->last) <= 0)) {
        s->first = new->first;
        break;
      }
    }

    if (!s) {
      imap_sequence *c;

      c = imap_sequence_new();
      c->first = new->first;
      c->last = new->last;
      c->next = seq;
      seq = c;
    }

    new = new->next;
  }

  return seq;
}


imap_sequence *imap_sequence_add_one(imap_sequence *seq,
                                     unsigned long message) {
  imap_sequence one;

  one.first = one.last = message;
  one.next = NULL;

  return imap_sequence_add(seq, &one);
}


void imap_sequence_to_buffer_stream(dbd_buffer_stream *bs,
                                    imap_sequence *seq) {
  for (; seq; seq = seq->next) {
    if (seq->first == 0) {
      dbd_buf_putc(bs, '*');
    } else 
      dbd_buf_printf(bs, "%d", seq->first);

    if (seq->first != seq->last) {
      if (seq->last == 0)
        dbd_buf_puts(bs, ":*");
      else
        dbd_buf_printf(bs, ":%d", seq->last);
    }

    if (seq->next) {
      dbd_buf_putc(bs, ',');
    }
  }
}
