#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <libdbd.h>

#include "imapproxy.h"
#include "login.h"
#include "imap_cmd.h"
#include "imap_resp.h"

struct cb_login_data {
  char *user, *passphrase;
};  

static int cb_login(imap_session *sess, char *tag, char *resp,
                    struct cb_login_data *cb);

int cmd_LOGIN(imap_session *sess, char *tag, char *cmd) {
  char *p;
  static struct cb_login_data cb = { NULL, NULL };

  if (sess->opt.login_disabled) {
    client_writeresp(sess, tag, "BAD Login disabled, send STARTTLS first");
    return 1;
  } 
  
  server_writecmd(sess, tag, cmd);

  cmd += 6; /* "LOGIN " */

  if ((p = resp_getstr(sess, cmd, &cb.user)) == NULL) {
    if ((p = strchr(cmd, ' ')) == NULL)
      goto error;
    cb.user = dbd_strndup(cmd, p - cmd);
    cmd = p + 1;
  } else
    cmd = p;

  if ((p = resp_getstr(sess, cmd, &cb.passphrase)) == NULL) {
    cb.passphrase = strdup(cmd);
    cmd = p + 1;
  } else
    cmd = p;

  resp_callback(sess, tag, cb_login, &cb);

  return 1;

  error:

  if (cb.user)       free(cb.user);
  if (cb.passphrase) free(cb.passphrase);

  return 1;  
}

static int cb_login(imap_session *sess, char *tag, char *resp,
                    struct cb_login_data *cb) {
  CryptContext *cc;
  char buf[BUFSIZE];

  if (strncmp(resp, "OK", 2) != 0) {
    client_writeresp(sess, tag, resp);
    return 1;
  }

  sess->state = IMAP_STATE_AUTHENTICATED;
  
  cc  = crypt_context_new();
  if (crypt_login(cc, cb->user, cb->passphrase) == -1) {
    snprintf(buf, sizeof (buf), "NO Crypt login failed: %s",
             crypt_strerror(cc));
    client_writeresp(sess, tag, buf);

    free(cb->user);       cb->user = NULL;
    free(cb->passphrase); cb->passphrase = NULL;
    crypt_context_destroy(cc);
      
    return 0; /* !! we should restart the server rather than hanging up,
                 because the imap server authenticated them, there
                 probably wasn't a typo */
  }

  mime_crypt_associate_pgp(sess->mc, cc);
  
  if (cb->user) {
    fixup_context_set_user(sess->fixup, cb->user);
    if (sess->user) free(sess->user);
    sess->user = dbd_strdup(cb->user);
  }
  
  free(cb->user);       cb->user = NULL;
  free(cb->passphrase); cb->passphrase = NULL;

  snprintf(buf, sizeof (buf), "%s; crypt login successful", resp);
  client_writeresp(sess, tag, buf);

  return 1;
}
