#ifndef MIME_CACHE_H
#define MIME_CACHE_H

typedef struct _imap_cache_folder {
  char folder[256];
  unsigned long folder_hash;
  unsigned long uidvalidity;
  unsigned long exists;
  unsigned long uids_total;
  unsigned long *uids;
  int hits, misses;
} imap_cache_folder;


typedef struct _imap_cache_msg {
  struct _imap_cache_folder *folder;
  unsigned long uid, msgno;

  struct _MimeContext *mime;

  struct _imap_cache_msg *older, *newer;
  struct _imap_cache_msg *next;
} imap_cache_msg;


typedef struct _imap_cache {
  int maxsize, size;

  int nfolders;
  imap_cache_folder **folders;
  
  int nbuckets;
  imap_cache_msg **buckets;
  imap_cache_msg *oldest, *newest;

  int hits, misses, expires;
} imap_cache;


extern void imap_cache_add_msg(imap_session *sess, char *folder,
                               unsigned long uidvalidity, unsigned long msgno,
                               unsigned long uid, struct _MimeContext *mime);
extern struct _MimeContext *imap_cache_get_msg(imap_session *sess,
                                               char *folder,
                                               unsigned long uidvalidity,
                                               unsigned long msgno,
                                               int by_uid);
extern void imap_cache_del_msg(imap_session *sess, char *folder,
                               unsigned long uidvalidity,
                               unsigned long msgno, int by_uid);
void imap_cache_expunge_msg(imap_session *sess, char *folder,
                            unsigned long uidvalidity, unsigned long msgno);
extern void imap_cache_init(imap_session *sess, int maxsize);
void imap_cache_finish(imap_session *sess);

#endif /* !MIME_CACHE_H */
