#include <string.h>
#include <stdio.h>

#include <openssl/ssl.h>
#include <openssl/err.h>

#include "imapproxy.h"

/* STARTTLS <CRLF> */
int cmd_STARTTLS(imap_session *sess, char *tag, char *cmd) {
  SSL_CTX *ctx = NULL;
  unsigned long ssl_err;
  X509 *client_cert;

  if (!sess->opt.starttls) {
    client_writeresp(sess, tag, "BAD TLS not available");
    return 1;
  }

      /* !! This might work, it hasn't been tested */
  
  if (sess->state != IMAP_STATE_UNAUTHENTICATED) {
    client_writeresp(sess, tag, "BAD Not while authenticated");
    return 1;
  }

  if (sess->tls) {
    client_writeresp(sess, tag, "BAD Already started TLS");
    return 1;
  }
  
  if ((ctx = SSL_CTX_new(TLSv1_server_method())) == NULL) {
    client_writeresp(sess, tag, "NO TLS not available: unable to initialize context");
    goto fail;
  }

  if (sess->opt.tls_key &&
      !SSL_CTX_use_PrivateKey_file(ctx, sess->opt.tls_key, SSL_FILETYPE_PEM)) {
    client_writeresp(sess, tag, "NO TLS not available: private key not available");
    goto fail;
  }
  
  if (sess->opt.tls_cert &&
      !SSL_CTX_use_certificate_file(ctx, sess->opt.tls_cert, SSL_FILETYPE_PEM)) {
    client_writeresp(sess, tag, "NO TLS not available: certificate not available");
    goto fail;
  }

  if ((sess->c_ssl = SSL_new(ctx)) == NULL) {
    client_writeresp(sess, tag, "NO TLS not available: unable to initialize ssl");
    goto fail;
  }

  dbd_log(LOG_ERR, "WARNING - Someone's using STARTTLS, "
          "email kaj@dbd.com with results");
  client_writeresp(sess, tag, "OK Begin TLS negotiation now (UNTESTED)");
  
  SSL_set_fd(sess->c_ssl, sess->c_sock);
  if ((ssl_err = SSL_accept(sess->c_ssl)) == -1) {
    char ssl_err_buf[BUFSIZE];
    dbd_log(LOG_ERR, "WARNING - SSL_accept failed in cmd_STARTTLS: %s",
           ERR_error_string(ssl_err, ssl_err_buf));
    
    goto fail;
  }

  if ((client_cert = SSL_get_peer_certificate(sess->c_ssl)) == NULL)
    dbd_log(LOG_ERR, "INFO - TLS started, client has no certificate");
  else {
    char subject[BUFSIZE], issuer[BUFSIZE];

    X509_NAME_oneline(X509_get_subject_name(client_cert), subject,
                      sizeof (subject));
  
    X509_NAME_oneline(X509_get_issuer_name(client_cert), issuer,
                      sizeof (issuer));

    dbd_log(LOG_ERR, "INFO - TLS started certificate issued for %s by %s",
           subject, issuer);

    X509_free(client_cert);
  }

  sess->tls = 1;
  
  return 1;

  fail:
  if (ctx) SSL_CTX_free(ctx);
  if (sess->c_ssl) {
    SSL_free(sess->c_ssl);
    sess->c_ssl = NULL;
  }

  return 1;
}
