#include "imapproxy.h"

void uidmap_add(imap_session *sess, unsigned long msgno, unsigned long uid) {
  if (sess->uidmap.alloced < msgno) {
    int a = sess->uidmap.alloced ? sess->uidmap.alloced : 1;

    while (a < msgno)
      a += a;

    sess->uidmap.uidmap =
      dbd_realloc(sess->uidmap.uidmap, a * sizeof (sess->uidmap.uidmap[0]));
    sess->uidmap.alloced = a;
  }

  sess->uidmap.uidmap[msgno] = uid;

  if (sess->uidmap.used < msgno)
    sess->uidmap.used = msgno;
}


void uidmap_expunge(imap_session *sess, unsigned long msgno) {
  if (sess->uidmap.used < msgno)
    return;

  if (sess->uidmap.used != msgno)
    memcpy(&sess->uidmap.uidmap[msgno], &sess->uidmap.uidmap[msgno+1],
           sizeof (unsigned long) * (sess->uidmap.used - msgno));

  sess->uidmap.uidmap[sess->uidmap.used] = 0;
  sess->uidmap.used--;
}


void uidmap_clear(imap_session *sess) {
  bzero(sess->uidmap.uidmap, sizeof (unsigned long) * sess->uidmap.used);
  sess->uidmap.used = 0;
}


unsigned long uidmap_by_msgno(imap_session *sess, unsigned long msgno) {
  if (sess->uidmap.used >= msgno)
    return sess->uidmap.uidmap[msgno];
  
  return 0;
}


unsigned long uidmap_by_uid(imap_session *sess, unsigned long uid) {
  int lo=1, hi=sess->uidmap.used;

  while (hi >= lo) {
    int mid;

        /* skip holes */
    for (mid = (hi+lo)/2; !sess->uidmap.uidmap[mid] && (mid <= hi); mid++)
          /* nuthin */;
    if (mid > hi) {
      for (mid = (hi+lo)/2; !sess->uidmap.uidmap[mid] && (mid >= lo); mid--)
            /* nuthin */;
    }
    if (mid < lo)
      return 0; /* exhausted */

    if (sess->uidmap.uidmap[mid] == uid)
      return sess->uidmap.uidmap[mid];
    else if (sess->uidmap.uidmap[mid] < uid)
      lo = mid+1;
    else if (sess->uidmap.uidmap[mid] > uid)
      hi = mid-1;
  }

  return 0;
}
