#include "libdbd.h"

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

long dbd_buf_write(dbd_buffer_stream *bs, const char *string, 
    unsigned long len) {
  unsigned long available;

  available = bs->size - bs->used;
  while(available < len) {
    if(available < 1024*1024) 
      bs->size *= 2;
    else 
      bs->size += 1024*1024;
    bs->buf = dbd_realloc(bs->buf, bs->size);
    available = bs->size - bs->used;
  }
  memcpy(bs->buf + bs->used, string, len);
  bs->used += len;
  return 1;
}

long dbd_buf_puts(dbd_buffer_stream *bs, const char *s) {
  return dbd_buf_write(bs, s, strlen(s));
}

long dbd_buf_puts_multiple(dbd_buffer_stream *bs, const char *s, ...) {
  va_list ap;
  const char *str;

  dbd_buf_write(bs, s, strlen(s));
  va_start(ap, s);
  while((str = va_arg(ap, const char *)) != NULL)
    dbd_buf_write(bs, str, strlen(str));
  return 1;
}

void dbd_buf_reserve(dbd_buffer_stream *bs, unsigned long size) {
  unsigned long available;

  available = bs->size - bs->used;
  while(available < size) {
    if(available < 1024*1024) 
      bs->size *= 2;
    else 
      bs->size += 1024*1024;
    bs->buf = dbd_realloc(bs->buf, bs->size);
    available = bs->size - bs->used;
  }
}


long dbd_buf_printf(dbd_buffer_stream *bs, const char *fmt, ...) {
  va_list ap;
  long ret;

  va_start(ap, fmt);
  ret = dbd_buf_vprintf(bs, fmt, ap);
  va_end(ap);

  return ret;
}


#if 1 /* glibc 2.1 *snprintf returns the lenght of the entire formatted
       * string even if it hits the end.  older versions return -1 on
       * overflow, presumably other libcs might as well */
long dbd_buf_vprintf(dbd_buffer_stream *bs, const char *fmt, va_list ap) {
  unsigned long available, size;
  va_list old_ap;

  __va_copy(old_ap, ap);

  available = bs->size - bs->used;
  size = vsnprintf(bs->buf+bs->used, available, fmt, ap);
  __va_copy(ap, old_ap);

  if (available < size) {
    while(available < size) {
      if(available < 1024*1024) 
        bs->size *= 2;
      else 
        bs->size += 1024*1024;
      bs->buf = dbd_realloc(bs->buf, bs->size);
      available = bs->size - bs->used;
    }
    vsnprintf(bs->buf+bs->used, available, fmt, ap);
  }

  bs->used += size;

  return 1;
}
#else
long dbd_buf_vprintf(dbd_buffer_stream *bs, const char *fmt, va_list ap) {
  unsigned long available, size;
  va_list old_ap;

  __va_copy(old_ap, ap);

  available = bs->size - bs->used;
  while ((size = vsnprintf(bs->buf+bs->used, available, fmt, ap)) == -1) {
    __va_copy(ap, old_ap);
    if(available < 1024*1024) 
      bs->size *= 2;
    else 
      bs->size += 1024*1024;
    bs->buf = dbd_realloc(bs->buf, bs->size);
    available = bs->size - bs->used;
  }

  bs->used += size;

  return 1;
}
#endif

char *dbd_buf_get_token(dbd_buffer_stream *bs, char delim) {
  char *p;
  char *end;

  p = bs->buf;
  end = p + bs->used;
  while(p < end) {
    if(*p == delim) {
      unsigned long len = p - bs->buf + 1;
      char *ret = dbd_malloc(len + 1);
      memcpy(ret, bs->buf, len);
      ret[len] = '\0';
      dbd_buf_discard(bs, len);
      return ret;
    }
    p++;
  }
  return NULL;
}

void dbd_buf_discard(dbd_buffer_stream *bs, unsigned long len) {
  memmove(bs->buf, bs->buf + len, bs->used - len);
  bs->used -= len;
}


