#include <libdbd.h>

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

static int syslog_opened = 0;
extern FILE *dbd_opt_log_handle;

#define LOG_BUFSIZE 4096

void dbd_log_func(int severity, const char *file, int line, 
    const char *format, ...) {
  va_list ap;
  dbd_buffer_stream bs;

  dbd_buf_init(&bs);
  
  if(dbd_opt_log_type == DBD_OPT_LOG_FILE) {
    time_t t;
    struct tm ts;

    if(severity > dbd_opt_log_severity) 
      return;

    if(!dbd_opt_log_handle)
      dbd_opt_log_handle = stderr;

    /* Assemble the log message in a buffer */
    t = time(NULL);
    localtime_r(&t, &ts);
    dbd_buf_printf(&bs, "%2d/%02d/%d %02d:%02d:%02d %s [%d]: ",
                   ts.tm_mon+1, ts.tm_mday, ts.tm_year+1900, ts.tm_hour, 
                   ts.tm_min, ts.tm_sec, dbd_opt_log_identity, getpid());
  }
    
  if(dbd_opt_log_lines)
    dbd_buf_printf(&bs, "%s line %d: ", file, line);

  va_start(ap, format);
  dbd_buf_vprintf(&bs, format, ap);
  va_end(ap);

  if(dbd_opt_log_type == DBD_OPT_LOG_FILE) {
    dbd_buf_putc(&bs, '\n');

    fwrite(bs.buf, 1, bs.used, dbd_opt_log_handle);
    fflush(dbd_opt_log_handle);
  }
  else {
    if(!syslog_opened) {
      openlog(dbd_opt_log_identity, LOG_PID, dbd_opt_log_facility);
      syslog_opened = 1;
    }
    dbd_buf_stringify(&bs);
    syslog(severity, "%s", bs.buf);
  }

  dbd_buf_free(&bs);
}

