#include <libdbd.h>

#include <stdio.h>
#include <stdlib.h>

/* This macro defines integer parameters. */
#define opt_int(token, def) \
  void dbd_opt_set_##token(int n) { \
    dbd_opt_##token = n; \
  } \
  int dbd_opt_##token = def

/* Ditto for char *. Note we need heap management for these. */
#define opt_char(token, def) \
  static int heap_used_##token = 0; \
  void dbd_opt_set_##token(const char *s) { \
    if(heap_used_##token) \
      free(dbd_opt_##token); \
    dbd_opt_##token = dbd_strdup(s); \
    heap_used_##token = 0; \
  } \
  char *dbd_opt_##token = def

/* maximum number of fork attempts before failure */
opt_int(max_fork_attempts, 5); 

/* timeout in seconds between fork attempts */
opt_int(fork_timeout, 2);

/* This defines behavior on malloc failure */
opt_int(failed_malloc, DBD_OPT_FAILURE_EXIT);

/*  Ditto for forking. */
opt_int(failed_fork, DBD_OPT_FAILURE_IGNORE);

/* This is the failure exit code if any libdbd action triggers
 * DBD_OPT_FAILURE_EXIT */
opt_int(failure_code, 1);

/* This is the most insignificant level that should be recorded. */
opt_int(log_severity, LOG_INFO);

/* This is the syslog facility to use (if syslogging is being used */
opt_int(log_facility, LOG_LOCAL0);

/* This should be 1 to log source filenames/line numbers, 0 otherwise */
opt_int(log_lines, 1);

/* This sets the location of logging. */
opt_int(log_type, DBD_OPT_LOG_FILE);

/* And this controls the file type */
FILE *dbd_opt_log_handle = NULL;
void dbd_opt_set_log_file(const char *file) {
  dbd_opt_log_type = DBD_OPT_LOG_FILE;

	if(file[0] == '-' && file[1] == '\0') {
    if(dbd_opt_log_handle && dbd_opt_log_handle != stderr)
      fclose(dbd_opt_log_handle);
		dbd_opt_log_handle = stderr;
		dbd_log(LOG_NOTICE, "Logging successfully started to stderr");
		return;
	}
	else {
		FILE *f;

		f = fopen(file, "a");
		if(!f) {
			dbd_error(LOG_EMERG, "Unable to open logging file %s", file);
			return;
		}
    
    if(dbd_opt_log_handle && dbd_opt_log_handle != stderr)
      fclose(dbd_opt_log_handle);
	
		dbd_opt_log_handle = f;
		dbd_log(LOG_NOTICE, "Logging successfully started to %s", file);
	}
}

opt_char(log_identity, "libdbd");

