#ifndef DBDOPTS_H
#define DBDOPTS_H

/* These three define the possible courses of action when libdbd
 * catches an error condition. */
#define DBD_OPT_FAILURE_IGNORE 1
#define DBD_OPT_FAILURE_EXIT 2
#define DBD_OPT_FAILURE_THREAD_EXIT 3

/* These define the various types of logging. */
#define DBD_OPT_LOG_FILE 1
#define DBD_OPT_LOG_SYSLOG 2

#define dbd_opt_decl(type, token) \
  extern type dbd_opt_##token; \
  extern void dbd_opt_set_##token(const type n)

/* maximum number of fork attempts before failure */
dbd_opt_decl(int, max_fork_attempts); 

/* timeout in seconds between fork attempts */
dbd_opt_decl(int, fork_timeout);

/* This defines behavior on malloc failure */
dbd_opt_decl(int, failed_malloc);

/*  Ditto for forking. */
dbd_opt_decl(int, failed_fork);

/* This is the failure exit code if any libdbd action triggers
 * DBD_OPT_FAILURE_EXIT */
dbd_opt_decl(int, failure_code);

/* This is the most insignificant level that should be recorded. */
dbd_opt_decl(int, log_severity);

/* This is the syslog facility to use (if syslogging is being used */
dbd_opt_decl(int, log_facility);

/* This should be 1 to log source filenames/log numbers, 0 otherwise */
dbd_opt_decl(int, log_lines);

/* This defines the identity string used in logging. */
dbd_opt_decl(char*, log_identity);

/* This is the type of logging used */
dbd_opt_decl(int, log_type);

/* This sets the file to be used for logging. */
void dbd_opt_set_log_file(const char *s);

#endif /* DBDOPTS_H */
