#include <libdbd.h>

#include <stdio.h>

void show_element(dbd_conf_element_t *e, FILE *out) {
  if(!e)
    return;

  fprintf(out, "%s = %s\n", e->key, e->value ? e->value : "NULL");
  show_element(e->next, out);
}

void show_section(dbd_conf_section_t *s, FILE *out) {
  if(!s)
    return;

  fprintf(out, "Section %s has %d elements:\n", s->name, s->size);

  show_element(s->elements, out);

  show_section(s->next, out);
}

void show_conf(dbd_conf_t *c, FILE *out) {
  fprintf(out, "Configuration file has %d sections:\n", c->size);
  show_section(c->sections, out);
}

void test_find(const dbd_conf_t *c, FILE *out, const char *section,
    const char *key) {
  const char *v;

  if(section)
    fprintf(out, "Attempting to find \"%s\" in section \"%s\"...\n", key, 
        section);
  else
    fprintf(out, "Attempting to find \"%s\" in any section...\n", key);

  v = dbd_conf_find(c, section, key);
  if(!v)
    fprintf(out, "Not found.\n");
  else
    fprintf(out, "%s\n", v);
}

int main() {
  dbd_conf_t *c;

  c = dbd_conf_parse_file("test-conf.in");
  if(!c)
    return 1;

  printf("Testing lookup functionality...\n\n");
  test_find(c, stdout, "Section 1", "foo");
  test_find(c, stdout, "Section with whitespace", "foo");
  test_find(c, stdout, NULL, "foo");
  test_find(c, stdout, "sEcTiOn 1", "FOO");
  test_find(c, stdout, NULL, "more");
  test_find(c, stdout, NULL, "not present");

  printf("\nEnumerating structure...\n\n");
  show_conf(c, stdout);
  return 0;
}



  

