#include "libdbd.h"
#include "dbdlog.h"

#include <stdio.h>
#include <stdlib.h>

#define FILE_TO_OPEN "/dev/kmem"

int main(int argc, char **argv) {
	char *p;
	pid_t pid;
	dbd_vec_t v;
	int i;
	char *s;
	char *tokens[64];
	int ntokens;

	if(argc < 2) {
		printf("Usage:\n%s <mem to allocate>\n", argv[0]);
		exit(1);
	}

	dbd_opt_set_log_identity("test-dbd");

	p = dbd_malloc(atoi(argv[1]));
	printf("Malloc'd to %p\n", p);
	
	printf("Forking...\n");
	pid = dbd_fork();
	if(pid) {
		printf("I'm the parent process!\n");
	}
	else {
		printf("I'm the child process!\n");
		exit(0);
	}
	
	printf("Enter a line of text:\n");
	fgets(p, atoi(argv[1]), stdin);

	printf("dbd_split test (w/ allownull):\n");
	s = dbd_strdup(p);
	dbd_chomp(s);
	ntokens = dbd_split(" \t", s, 64, tokens, 1);
	printf("I received %d tokens.\n", ntokens);
	printf("They were:");
	for(i = 0; i < ntokens; i++) {
		fputs(" \"", stdout);
		fputs(tokens[i], stdout);
		fputc('"', stdout);
	}
	fputc('\n', stdout);
	free(s);
	
	printf("dbd_split test (w/o allownull):\n");
	s = dbd_strdup(p);	
	dbd_chomp(s);
	ntokens = dbd_split(" \t", s, 64, tokens, 0);
	printf("I received %d tokens.\n", ntokens);
	printf("They were:");
	for(i = 0; i < ntokens; i++) {
		fputs(" \"", stdout);
		fputs(tokens[i], stdout);
		fputc('"', stdout);
	}
	fputc('\n', stdout);

	printf("dbd_split_a test:\n");
	v = dbd_split_a(" \t\r\n", p);
	printf("I received %d tokens.\n", v.len);
	printf("They were:");
	for(i = 0; v.v[i]; i++) {
		fputs(" \"", stdout);
		fputs(v.v[i], stdout);
		fputc('"', stdout);
	}
	fputc('\n', stdout);
	
	fputs("Freeing vector...\n", stdout);
	dbd_vecfree(v);

	dbd_log(LOG_ERR, "I'm now %s with severity %d", "logging", 
			LOG_ERR);
	fopen(FILE_TO_OPEN, "ra");
	dbd_error(LOG_ERR, "Attempt to open %s", FILE_TO_OPEN);
	dbd_log(LOG_DEBUG, "This is a debug log.");
	return 0;

}
