#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <libdbd.h>

#include "spewery.h"

static spew_block *spew_new(void);
static int spew_dict_find_var(spew_dict *dict, char *name);

static spew_block *spew_new(void) {
  spew_block *new;

  bzero(new = dbd_malloc(sizeof (spew_block)), sizeof (spew_block));

  return new;
}

spew_block *spew_new_from_buffer(char *buf) {
  spew_block *spew = NULL, *spewt = NULL, *b;
  char *p = buf, *q;

  while (p && *p) {
    for (q = strchr(p, '$'); q; q = strchr(q, '$')) {
      if (q[1] != '$')
        break;
    }
    if (q == NULL)
      q = p + strlen(p);

    if (q > p) {
      b = spew_new();
      b->type = SPEW_TEXT;
      b->data.text = dbd_memstr(p, q - p);

      if (spewt)
        spewt->next = b;
      else
        spew = b;
      spewt = b;    
    }

    if (*q != '\0') {
      q++;

      if (*q == '{') {
        q++;
        if ((p = strchr(q, '}')) != NULL) {
          b = spew_new();
          b->type = SPEW_VAR;
          b->data.var.name = dbd_memstr(q, p - q);

          if ((q = strchr(b->data.var.name, '-')) != NULL) {
            *q = '\0';
            q++;
            b->data.var.dfault = dbd_strdup(q);
          }

          if (spewt)
            spewt->next = b;
          else
            spew = b;
          spewt = b;        

          p++;
        } else { /* no closing brace, just display it as is */
          b = spew_new();
          b->type = SPEW_TEXT;
          b->data.text = dbd_memstr(q - 2, strlen(q - 2));

          if (spewt)
            spewt->next = b;
          else
            spew = b;
          spewt = b;        

          p = NULL;
        }
      } else {
        for (p = q; *p && (isalnum(*p) || (*p == '_')); p++)
              /* nuthin */;

        b = spew_new();
        b->type = SPEW_VAR;
        b->data.var.name = dbd_memstr(q, p - q);

        if (spewt)
          spewt->next = b;
        else
          spew = b;
        spewt = b;        
      }
    } else
      p = NULL;
  }

  return spew;
}


spew_dict *spew_dict_new(void) {
  spew_dict *dict;

  bzero(dict = dbd_malloc(sizeof (spew_dict)), sizeof (spew_dict));

  return dict;
}


static int spew_dict_find_var(spew_dict *dict, char *name) {
  int lo = 0, hi = dict->size - 1;

  while (lo <= hi) {
    int cmp, mid = (lo + hi) / 2;

    if ((cmp = strcasecmp(name, dict->entries[mid]->name)) == 0)
      return mid;
    else if (cmp < 0)
      hi = mid - 1;
    else if (cmp > 0)
      lo = mid + 1;
  }

  return -1;
}


void spew_dict_add_vars(spew_dict *dict, ...) {
  va_list ap, old_ap;
  char *name, *value;
  int i = 0;

  __va_copy(old_ap, ap);
  va_start(ap, dict);
  while (((name  = va_arg(ap, char *)) != NULL) &&
         ((value = va_arg(ap, char *)) != NULL)) {
    if (spew_dict_find_var(dict, name) == -1)
      i++;
  }
  va_end(ap);
  __va_copy(ap, old_ap);

  if (i)
    dict->entries =
      dbd_realloc(dict->entries,
                  (dict->size + i) * (sizeof (spew_dict_entry)));
  
  va_start(ap, dict);
  while (((name  = va_arg(ap, char *)) != NULL) &&
         ((value = va_arg(ap, char *)) != NULL)) {
    if ((i = spew_dict_find_var(dict, name)) == -1) {
      int cmp = -1, mid = 0, lo = 0, hi = dict->size - 1;

      while (lo <= hi) {
        mid = (lo + hi) / 2;
        
        if ((cmp = strcasecmp(name, dict->entries[mid]->name)) == 0) {
          i = mid;
          break;
        } else if (cmp < 0)
          hi = mid - 1;
        else if (cmp > 0)
          lo = mid + 1;
      }

      if (i == -1) {
        if (cmp < 0)
          i = mid;
        else
          i = mid + 1;
      }
      
      memmove(&dict->entries[i+1], &dict->entries[i],
              (dict->size - i) * sizeof (spew_dict_entry));

      dict->entries[i] = dbd_malloc(sizeof (spew_dict_entry));
      dict->entries[i]->name = dbd_strdup(name);
      dict->entries[i]->value = dbd_strdup(value);

      dict->size++;
    } else {
      free(dict->entries[i]->value);
      dict->entries[i]->value = dbd_strdup(value);
    }
  }

  va_end(ap);
}


void spew_to_buffer(dbd_buffer_stream *bs, spew_block *spew, spew_dict *dict) {
  int i;
  
  while (spew) {
    switch (spew->type) {
        case SPEW_TEXT :
          dbd_buf_puts(bs, spew->data.text);
          break;
        case SPEW_VAR :
          if ((i = spew_dict_find_var(dict, spew->data.var.name)) != -1)
            dbd_buf_puts(bs, dict->entries[i]->value);
          else if (spew->data.var.dfault)
            dbd_buf_puts(bs, spew->data.var.dfault);
          break;
    }

    spew = spew->next;
  }
}


void spew_destroy(spew_block *spew) {
  while (spew) {
    switch (spew->type) {
        case SPEW_TEXT :
          free(spew->data.text);
          break;
        case SPEW_VAR :
          free(spew->data.var.name);
          break;
    }

    spew = spew->next;
  }
}

void spew_dict_destroy(spew_dict *dict) {
  while (dict->size) {
    free(dict->entries[dict->size - 1]->name);
    free(dict->entries[dict->size - 1]->value);
    free(dict->entries[dict->size - 1]);
  }

  if (dict->entries)
    free(dict->entries);

  free(dict);
}


  

    
