#include <mimeboundary.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

char *mime_boundary_generate(char *buf) {
  static long int hostid = 0;

  if (!hostid)
    hostid = gethostid();
  
  sprintf(buf, "%ld-%ld-%ld=:%ld", hostid,
      (long)random(), (long)time(0), (long)getpid());
  return buf;
}

/* Find the boundary located in a buffer. 
 * Returns -1 in case of end of buffer
 * Returns 0 in case the delimiter found is the closing delimiter
 * Returns 1 otherwise */
int mime_boundary_find_in_buf(const char *buf, unsigned long len,
    const char *boundary, const char **result_begin, 
    const char **result_end) {
  const char *end = buf + len;
  int matched = 2; /* we start out having matched \r\n in case there
                      is no prologue at all */
  int looking_for = '-';
  const char *p;
  const char *start = NULL;
  unsigned long bsize = strlen(boundary);

  for(p = buf; p < end; p++) {
    if(*p == looking_for) {
      matched++;
      if(matched == 1) {
        /* We found the CR, mark the start and look for newline. */
        start = p;
        looking_for = '\n';
      }
      else if(matched == 2 || matched == 3) {
       /* We found the CRLF, now find the dashes. */
       looking_for = '-';
      }
      else if(matched < bsize + 4) {
        /* Otherwise look for the next character in the boundary. */
        looking_for = boundary[matched - 4];
      }
      else {
        /* We matched the whole boundary. Break. */
        break;
      }
    }
    /* If it's not what we're looking for, reset matched */
    else {
      if(*p == '\r') {
        start = p;
        matched = 1;
        looking_for = '\n';
      }
      else {
        matched = 0;
        looking_for = '\n';
      }
    }
  }

  /* If we didn't find anything useful, return NULL. This should never
   * happen with well-formed MIME, but you never know. */
  if(p == end)
    return -1;

  /* So here we have the boundary matched but don't know what's after it. 
   * Increment p to the next character for examination. */
  p++;
  if(end - p > 1 && *p == '-' && *(p+1) == '-') {
    /* This is the close delimiter. Return end. */
    *result_begin = start;
    *result_end = p + 4;
    return 0;
  }
  /* Otherwise, let's make sure we have whitespace followed by CRLF. */
  else {
    while(p < end) {
      /* Skip whitespace. */
      if(*p == ' ' || *p == '\t')
        p++;
      else if(*p == '\r' && end - p > 1 && *(p+1) == '\n') {
        /* This is correct. We found it. */
        *result_begin = start;
        *result_end = p+2;
        return 1;
      }
      else {
        /* Just barely not right. Look in the rest of the buffer. */
        return mime_boundary_find_in_buf(p, end - p, boundary, result_begin,
            result_end);
      }
    }
    /* Oops. We hit the end. */
    return -1;
  }
  /* should not be reached. */
}

