#include <mimebuffer.h>

#include <libdbd.h>

#include <stdlib.h>

MimeBuffer *mime_buffer_new_with_value(char *buf, unsigned long len) {
  MimeBuffer *m;

  m = dbd_malloc(sizeof(MimeBuffer));

  m->buf = buf;
  m->len = len;
  m->sub = NULL;

  return m;
}

static MimeSubBuffer *mime_subbuffer_copy(const MimeSubBuffer *old) {
  MimeSubBuffer *sb;

  sb = dbd_malloc(sizeof(MimeSubBuffer));
  sb->len = old->len;
  sb->buf = dbd_malloc(sb->len);
  memcpy(sb->buf, old->buf, sb->len);

  if(old->next)
    sb->next = mime_subbuffer_copy(old);
  else
    sb->next = NULL;

  return sb;
}

MimeBuffer *mime_buffer_copy(const MimeBuffer *old) {
  MimeBuffer *m;

  m = dbd_malloc(sizeof(MimeBuffer));
  m->len = old->len;
  if(!old->buf)
    m->buf = NULL;
  else {
    m->buf = dbd_malloc(m->len);
    memcpy(m->buf, old->buf, m->len);
  }
  if(!old->sub)
    m->sub = NULL;
  else
    m->sub = mime_subbuffer_copy(old->sub);

  return m;
}

void mime_buffer_destroy(MimeBuffer *m) {
  MimeSubBuffer *s, *next;

  if(m->buf)
    free(m->buf);

  for(s = m->sub; s; s = next) {
    next = s->next;
    if(s->buf)
      free(s->buf);
    free(s);
  }

  free(m);
}

void mime_buffer_attach_subbuffer(MimeBuffer *m, char *buf, unsigned long len) {
  MimeSubBuffer *sb;

  sb = dbd_malloc(sizeof(MimeSubBuffer));
  sb->buf = buf;
  sb->len = len;

  sb->next = m->sub;
  m->sub = sb;
}

/* Ack. This is incredibly ugly.  Take a pointer to either the buffer
 * b or one of its subbuffers and return a similarly aligned pointer for
 * a. */
const char *mime_buffer_align(const MimeBuffer *a, const MimeBuffer *b, 
    const char *bp) {
  const MimeSubBuffer *sb;
  const char *bbuf;
  int n;

  /* Are we in the main buffer? */
  if(bp >= b->buf && bp < b->buf + b->len) {
    /* Yes. Return relative alignment. */
    return a->buf + (bp - b->buf);
  }

  /* Otherwise, we'll have to find the correct sub-buffer. */
  n = 0;
  sb = b->sub;
  while(sb && (bp < sb->buf || bp >= sb->buf + sb->len)) {
    sb = sb->next;
    n++;
  }

  /* Did we find it? */
  if(!sb) {
    /* If not, well...that shouldn't happen. */
    dbd_log(LOG_DEBUG, "libmime: Ack! I was outside of the buffer"
        " during a mime_buffer_align");
    return NULL;
  }

  /* Save the buf spot for alignment. */
  bbuf = sb->buf;

  /* It was in the nth sub-buffer. The subbuffer ordering should correspond
   * if they were copied using mime_buffer_copy() */
  sb = a->sub;
  while(n && sb) {
    sb = sb->next;
    n--;
  }

  if(!sb) {
    /* There is no nth subbuffer? Ack. These aren't really copies! */
    dbd_log(LOG_DEBUG, "libmime: Couldn't find the nth subbuffer "
        "(I was %d short)", n);
    return NULL;
  }

  /* Otherwise we've found it. Return the re-aligned buffer. */
  return sb->buf + (bp - bbuf);
}

