#ifndef MIMECRYPT_H
#define MIMECRYPT_H

#include <mimebody.h>
#include <libdbd.h>
#include <crypt-client.h>
#include <openssl/ssl.h>

/* This file consists of generic MIME-cryptographic functions. Mainly these 
 * exist to dispatch to the specific crypto functions (PGP, S/MIME, etc) 
 *
 * It also includes the MimeCrypt object, which abstracts the interface
 * to cryptographic interfaces. */

typedef struct _MimeCrypt MimeCrypt;
struct _MimeCrypt {
  CryptContext *pgp;
  X509 *smime_signer_cert;
  EVP_PKEY *smime_signer_key;
};

MimeCrypt *mime_crypt_new();
void mime_crypt_associate_pgp(MimeCrypt *mc, CryptContext *cc);
void mime_crypt_associate_smime(MimeCrypt *mc, X509 *cert, EVP_PKEY *key);
void mime_crypt_destroy(MimeCrypt *m);

int mime_crypt_encrypted_body_to_buffer_stream(const MimeBody *b, 
    dbd_buffer_stream *bs, MimeCrypt *cc, const char *boundary);

int mime_crypt_signed_body_to_buffer_stream(const MimeBody *b, 
    dbd_buffer_stream *bs, MimeCrypt *cc, const char *boundary);

void mime_crypt_encrypted_body_parse(MimeBody *b, MimeCrypt *cc);
void mime_crypt_signed_body_parse(MimeBody *b, MimeCrypt *cc);

void mime_crypt_parse_notables(MimeBody *b, 
    const DecodedMessageNotable *notables, unsigned nnotables);

MimeBody *mime_crypt_flatten_encrypted(MimeBody *b);

#endif /* MIMECRYPT_H */
