#ifndef ENCODING_H
#define ENCODING_H

#include <libdbd.h>

#define MIME_ENCODING_7BIT 0
#define MIME_ENCODING_8BIT 1
#define MIME_ENCODING_BINARY 2
#define MIME_ENCODING_BASE64 3
#define MIME_ENCODING_QUOTEDPRINTABLE 4
#define MIME_ENCODING_OTHER 5

/* Undo the base64 encoding on a buffer. The return value is 
 * newly allocated, and its length is in dest_len.  */
unsigned char *mime_encoding_unbase64(const unsigned char *buf, 
    unsigned long len, unsigned long *dest_len);

/* Undo the quoted printable encoding on a buffer. */
unsigned char *mime_encoding_unqprint(const unsigned char *buf, 
    unsigned long len, unsigned long *dest_len);

/* Look up the integer associated with a given encoding string */
int mime_encoding_from_string(const char *s);

/* Look up the string associated with a given encoding integer */
const char *mime_encoding_to_string(int n);

/* These functions encode from a buffer to a buffer_stream. */
void mime_encoding_qprint_to_buffer_stream(dbd_buffer_stream *bs,
    const unsigned char *buf, unsigned long len);
void mime_encoding_base64_to_buffer_stream(dbd_buffer_stream *bs,
    const unsigned char *buf, unsigned long len);

#endif /* ENCODING_H */
