#include <mimemessage.h>

#include <mimecontext.h>

#include <string.h>
#include <stdlib.h>

static MimeMessage *mime_message_new() {
  MimeMessage *m;

  m = dbd_malloc(sizeof(MimeMessage));
  m->envelope = NULL;
  m->body = NULL;

  return m;
}

MimeMessage *mime_message_new_blank() {
  MimeMessage *m;

  m = dbd_malloc(sizeof(MimeMessage));
  m->envelope = mime_envelope_new();
  m->body = NULL;

  return m;
}
  
MimeMessage *mime_message_copy(const MimeMessage *old, MimeContext *mc) {
  MimeMessage *m;

  m = dbd_malloc(sizeof(MimeMessage));
  if(old->envelope)
    m->envelope = mime_envelope_copy(old->envelope);
  else
    m->envelope = NULL;

  if(old->body)
    m->body = mime_body_copy(old->body, mc);
  else
    m->body = NULL;

  return m;
}

void mime_message_destroy(MimeMessage *m) {
  if(m->envelope)
    mime_envelope_destroy(m->envelope);
  if(m->body)
    mime_body_destroy(m->body);
  free(m);
}

MimeMessage *mime_message_new_from_buf(const char *message, unsigned long len,
    MimeContext *mc) {
  char *p;
  MimeMessage *ret;

  /* First find the break between header and body. */
  p = memmem(message, len, "\r\n\r\n", 4);
  /* If it doesn't exist, we're kind of screwed. Try looking for a double
   * newline just in case. If that's not there, assume that everything is
   * just headers with no body. If we do find it, point to the first
   * character in the body. */
  if(p)
    p += 4;
  else {
    p = memmem(message, len, "\n\n", 2);
    if(p)
      p += 2;
  }

  ret = mime_message_new();

  ret->envelope = mime_envelope_new_from_buf(message, p ? p - message : len);
  if(!ret->envelope) {
    mime_message_destroy(ret);
    return NULL;
  }

  if(p) {
    ret->body = mime_body_new_from_buf_with_envelope(p, len - (p - message),
        ret->envelope, mc);
    if(!ret->body) {
      mime_message_destroy(ret);
      return NULL;
    }
  }

  return ret;
}

int mime_message_to_buffer_stream(MimeMessage *mm, dbd_buffer_stream *bs,
    MimeCrypt *mc) {
  if(mm->envelope)
    if(!mime_envelope_to_buffer_stream(mm->envelope, bs))
      return 0;
  if(mm->body) {
    if(!mime_body_to_buffer_stream(mm->body, bs, mc))
      return 0;
  } else
    dbd_buf_puts(bs, "\r\n");

  return 1;
}

void mime_message_crypt_unpack(MimeMessage *mm, MimeCrypt *mc) {
  if(mm->body)
    mime_body_crypt_unpack(mm->body, mc);
}

void mime_message_crypt_flatten(MimeMessage *mm) {
  if(mm->body)
    mm->body = mime_body_crypt_flatten(mm->body);
}

int mime_message_header_to_buffer_stream(MimeMessage *mm,
                                         dbd_buffer_stream *bs) {
  if(mm->envelope)
    if(!mime_envelope_to_buffer_stream(mm->envelope, bs))
      return 0;
  if(mm->body)
    if(!mime_body_header_to_buffer_stream(mm->body, bs))
      return 0;
  
  return 1;
}

int mime_message_text_to_buffer_stream(MimeMessage *mm, dbd_buffer_stream *bs,
                                       MimeCrypt *mc) {
  if(mm->body)
    if(!mime_body_text_to_buffer_stream(mm->body, bs, mc))
      return 0;
  return 1;
}

