#ifndef MIMEMESSAGE_H
#define MIMEMESSAGE_H

#include <mimeenvelope.h>
#include <mimebody.h>
#include <libdbd.h>
#include <mimecrypt.h>

struct _MimeContext;

typedef struct _MimeMessage MimeMessage;
struct _MimeMessage {
  MimeEnvelope *envelope;
  MimeBody *body;
};

/* Decode a single message (in a buffer) into its mime parts. */
MimeMessage *mime_message_new_from_buf(const char *message, unsigned long len,
    struct _MimeContext *mc);

/* Create a new blank message (for message creation from scratch) */
MimeMessage *mime_message_new_blank();

/* Make a complete (no shared resources) copy of a message */
MimeMessage *mime_message_copy(const MimeMessage *old, struct _MimeContext *mc);

/* Destroy a mime message, freeing all allocated resources. */
void mime_message_destroy(MimeMessage *m);

int mime_message_to_buffer_stream(MimeMessage *m, dbd_buffer_stream *bs,
    MimeCrypt *mc);

/* Write out a message header, a combination of the envelope and the body
 * content header.  No trailing blank line */
int mime_message_header_to_buffer_stream(MimeMessage *mm,
                                         dbd_buffer_stream *bs);

/* Write out a message body, the same as mime_body_text_to_buffer_stream() */
int mime_message_text_to_buffer_stream(MimeMessage *mm, dbd_buffer_stream *bs,
                                       MimeCrypt *mc);

/* Decrypt any encrypted segments and verify any signatures. */
void mime_message_crypt_unpack(MimeMessage *m, MimeCrypt *mc);

/* Flattening removes multipart/encrypted in favor of its body part */
void mime_message_crypt_flatten(MimeMessage *m);

#endif /* MIMEMESSAGE_H */
