#include <mimeparameter.h>

#include <libdbd.h>

#include <stdlib.h>

MimeParameter *mime_parameter_new() {
  MimeParameter *p;

  p = dbd_malloc(sizeof(MimeParameter));
  p->key = p->value = NULL;
  p->next = NULL;

  return p;
}

MimeParameter *mime_parameter_new_with_value(const char *key,
                                             const char *value) {
  MimeParameter *p;

  p = dbd_malloc(sizeof(MimeParameter));
  p->key = dbd_strdup(key);
  p->value = dbd_strdup(value);
  p->next = NULL;

  return p;
}

void mime_parameter_destroy_list(MimeParameter *p) {
  MimeParameter *next;

  while(p) {
    next = p->next;
    if(p->key)
      free(p->key);
    if(p->value)
      free(p->value);
    free(p);
    p = next;
  }
}

MimeParameter *mime_parameter_copy(const MimeParameter *old) {
  MimeParameter *ret;

  if(!old)
    return NULL;

  ret = dbd_malloc(sizeof(MimeParameter));
  ret->key = dbd_strdup(old->key);
  ret->value = dbd_strdup(old->value);
  ret->next = mime_parameter_copy(old->next);

  return ret;
}

char *mime_parameter_destroy_to_value(MimeParameter *p) {
  char *ret = p->value;

  if(p->key)
    free(p->key);
  free(p);
  
  return ret;
}

const char *mime_parameter_find(const MimeParameter *p, const char *key) {
  while(p) {
    if(!strcasecmp(p->key, key))
      return p->value;
    p = p->next;
  }
  return NULL;
}
