#ifndef MIMETYPE_H
#define MIMETYPE_H

#include "libdbd.h"

/* These are in alphabetical order by type name
 * to facilitate searching. */
#define MIME_TYPE_APPLICATION  0
#define MIME_TYPE_AUDIO        1
#define MIME_TYPE_IMAGE        2
#define MIME_TYPE_MESSAGE      3
#define MIME_TYPE_MULTIPART    4
#define MIME_TYPE_TEXT         5
#define MIME_TYPE_VIDEO        6

/* This should always be last. */
#define MIME_TYPE_OTHER        7

typedef struct _MimeType MimeType;
struct _MimeType {
  int type;
  char *subtype;
  char *x_type;
};

MimeType *mime_type_new();
MimeType *mime_type_new_with_value(int type, const char *subtype);
MimeType *mime_type_new_from_string(const char *s);
MimeType *mime_type_new_text_plain();
MimeType *mime_type_copy(const MimeType *old);
MimeType *mime_type_new_from_filename(const char *filename);

void mime_type_destroy(MimeType *m);

/* Grab the main type out of a MimeType. This does a lookup
 * in a table to get the string corresponding to the type integer,
 * or if it is an unknown type, it returns the x_type string. */
const char *mime_type_get_type(const MimeType *m);

/* Output a pretty-printed mime type to the buffer stream. */
void mime_type_to_buffer_stream(const MimeType *m, dbd_buffer_stream *bs);

#define mime_type_is_rfc822(t) ((t)->type == MIME_TYPE_MESSAGE \
    && !strcasecmp((t)->subtype, "RFC822"))


extern const char *mime_types[];

#endif /* MIMETYPE_H */
