#include "mimetypedb.h"

#include <libdbd.h>
#include <mimedefaults.h>

typedef struct _MimeTypeDB MimeTypeDB;
struct _MimeTypeDB {
  char *extension;
  MimeType *type;
  struct _MimeTypeDB *left;
  struct _MimeTypeDB *right;
};

static MimeTypeDB *mime_type_database = NULL;

static MimeTypeDB *mime_type_database_new(const char *ext, const char *type) {
  MimeTypeDB *db;

  db = dbd_malloc(sizeof(MimeTypeDB));
  db->extension = dbd_strdup(ext);
  db->type = mime_type_new_from_string(type);
  db->left = db->right = NULL;

  return db;
}

static void mime_type_database_insert(const char *ext, const char *type) {
  MimeTypeDB *p;

  if(!mime_type_database) {
    mime_type_database = mime_type_database_new(ext, type);
    return;
  }

  p = mime_type_database;
  while(1) {
    int cmp = strcmp(ext, p->extension);

    if(cmp < 0) {
      if(!p->left) {
        p->left = mime_type_database_new(ext, type);
        return;
      }
      p = p->left;
    }
    else if(cmp > 0) {
      if(!p->right) {
        p->right = mime_type_database_new(ext, type);
        return;
      }
      p = p->right;
    }
    else {
      dbd_log(LOG_INFO, "mime-type DB: %s duplicates use of extension %s",
          type, ext);
      return;
    }
  }
}

void mime_type_database_init() {
  FILE *fp;
  char *buf = NULL;
  unsigned long len = 0;

  fp = fopen(mime_default_mime_types, "r");
  if(!fp)
    return;

  while(1) {
    dbd_vec_t v;
    int i;

    buf = dbd_getline(buf, &len, fp);
    if(!buf)
      return;

    dbd_chomp(buf);
    if(buf[0] == '#' || buf[0] == '\0')
      continue;

    v = dbd_split_a(" \t", buf);
    for(i = 1; i < v.len; i++) 
      mime_type_database_insert(v.v[i], v.v[0]);

    dbd_vecfree(v);
  }
}

const MimeType *mime_type_database_find(const char *extension) {
  MimeTypeDB *p;

  p = mime_type_database; 
  while(p) {
    int cmp;

    cmp = strcmp(extension, p->extension);
    if(cmp < 0)
      p = p->left;
    else if(cmp > 0)
      p = p->right;
    else
      return p->type;
  }
  return NULL;
}
