#include <stdio.h>

#include "libdbd.h"
#include "mimecontext.h"

int main(int argc, char **argv) {
  dbd_buffer_stream bs;
  MimeContext *mc, *copy;
  MimeCrypt *mime_crypt = NULL;
  
  if(argc > 2) {
    CryptContext *cc = crypt_context_new();

    mime_crypt = mime_crypt_new();
    mime_crypt_associate_pgp(mime_crypt, cc);

    if(crypt_login(cc, argv[1], argv[2]) < 0) {
      printf("failed crypt_login: %s\n", crypt_strerror(cc));
      return 1;
    }
    argv += 2;
  }

  dbd_buf_init(&bs);
  while(1) {
    char buf[2048];
    int result;

    result = fread(buf, 1, 2048, stdin);
    if(!result)
      break;
    dbd_buf_write(&bs, buf, result);
  }

  mc = mime_context_new_from_buf(bs.buf, bs.used);
  mime_context_crypt_unpack(mc, mime_crypt);

  copy = mime_context_copy(mc);
  mime_context_destroy(mc);
  mc = mime_context_copy(copy);
  mime_context_destroy(copy);

  if(argv[1]) {
    MimeBody *body;
    const char *buf;
    unsigned long len;
    const MimeSignature *ms;

    body = mime_context_get_body_by_number(mc, argv[1]);
    if(!body) {
      printf("Couldn't find body part %s\n", argv[1]);
      return 1;
    }
    buf = mime_body_get_data(body, &len);

    if(!buf)
      printf("Error getting body section");
    else
      fwrite(buf, 1, len, stdout);

    for(ms = body->signatures; ms; ms = ms->next) {
      fprintf(stderr, "Part %s was signed:\n", argv[1]);
      fprintf(stderr, "Time: %s", ctime(&ms->timestamp));
      if(ms->keyid)
        fprintf(stderr, "KeyID: %s\n", ms->keyid);
      if(ms->status == CRYPT_STATUS_UNKNOWN)
        fprintf(stderr, "The signer's key could not be located."
           " The signature is unverified.\n");
      else if(ms->status == CRYPT_STATUS_GOOD)
        fprintf(stderr, "The signature was verified correctly.\n");
      else 
        fprintf(stderr, "The signature was INCORRECT.\n");
      if(ms->name)
        fprintf(stderr, "Key Owner: %s\n", ms->name);
    }
  }
  else {
    char buf[4096];
    unsigned long len;
   
    len = mime_context_get_structure(mc, buf, 4096);
    fwrite(buf, 1, len, stdout);
  }

  mime_context_destroy(mc);

  return 0;
}
