#include <libdbd.h>
#include <libmime.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char **argv) {
  dbd_buffer_stream bs;
  MimeContext *mc;
  char *data;
  unsigned long datalen;
  MimeCrypt *mime_crypt;

  OpenSSL_add_ssl_algorithms();

  mime_crypt = mime_crypt_new();

  if(!argv[1]) {
    fprintf(stderr, "Require \"pgp\" or \"smime\" as first parameter.\n");
    return 1;
  }
  if(!strcasecmp(argv[1], "pgp")) {
    CryptContext *cc;

    if(!argv[2] || !argv[3]) {
      fprintf(stderr, "Require login and passphrase for crypt server on "
          "command line.\n");
      return 1;
    }

    cc = crypt_context_new();
    if(crypt_login(cc, argv[1], argv[2]) < 0) {
      fprintf(stderr, "failed crypt_login: %s\n", crypt_strerror(cc));
      return 1;
    }

    mime_crypt_associate_pgp(mime_crypt, cc);
  }
  else if(!strcasecmp(argv[1], "smime")) {
    X509 *cert;
    EVP_PKEY *key;
    BIO *in;

    if(!argv[2]) {
      fprintf(stderr, "Require signer certificate on command line.\n");
      return 1;
    }

    in = BIO_new_file(argv[2], "r");
    if(!in) {
      fprintf(stderr, "Couldn't open signer file %s\n", argv[2]);
      return 1;
    }

    cert = PEM_read_bio_X509(in, NULL, NULL, NULL);
    if(!cert) {
      fprintf(stderr, "Can't find valid cert in file %s\n", argv[2]);
      return 1;
    }

    BIO_reset(in);
    key = PEM_read_bio_PrivateKey(in, NULL, NULL, NULL);
    if(!key) {
      fprintf(stderr, "Can't find valid key in file %s\n", argv[2]);
      return 1;
    }

    BIO_free(in);

    mime_crypt_associate_smime(mime_crypt, cert, key);
  }
  else {
    fprintf(stderr, "Require \"pgp\" or \"smime\" as first parameter.\n");
    return 1;
  }

  dbd_buf_init(&bs);
  while(1) {
    char buf[2048];
    int result;

    result = fread(buf, 1, 2048, stdin);
    if(!result)
      break;
    dbd_buf_write(&bs, buf, result);
  }

  mc = mime_context_new_from_buf(bs.buf, bs.used);

  if(!strcasecmp(argv[1], "pgp"))
    mime_context_pgp_sign(mc);
  else
    mime_context_smime_sign(mc);

  data = mime_context_to_buffer(mc, &datalen, mime_crypt);
  if(!data)
    printf("Some sort of error or something.\n");
  else
    write(1, data, datalen);

  mime_context_destroy(mc);
  return 0;
}
