package      PGPPacket;
require      Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(pgppacketinfo pgpapacketinfo);
@EXPORT_OK = qw();

use SHA;
use MD5;
use MIME::Base64;

sub readpacket {
    my $data = shift;
    my $p = 0;
    my $pkt = {};
    my $tag;
    my $length;
    my $newfmt;
    my $partial;
    my $type;

    $pkt->{body} = '';
    do {
	unless (length($tag = substr($data, $p, 1)) == 1) {
	    return undef;
	}
	$p += 1;
	$tag = unpack("C", $tag);

	unless ($tag & 0x80) {
	    warn "1bit not set, invalid packet\n";
	    return undef;
	}

	$partial = 0;
	if ($newfmt = (($tag & 0x40) >> 6)) {
	    $type = $tag & 0x3F;
	    $length = substr($data, $p, 1); $p += 1;
	    $length = unpack("C", $length);
	    if ($length < 192) {
	    } elsif (($length >= 192) and ($length < 224)) {
		my $high = $length;
		my $low;
		$low = substr($data, $p, 1); $p += 1;
		$low = unpack("C", $low);
		$length = (($high - 192) << 8) + $low + 192;
	    } elsif (($length >= 224) and ($length < 255)) {
		$partial = 1;
		$length = 1 << ($length & 0x1F);
	    } elsif ($length == 255) {
		$length = substr($data, $p, 4); $p += 4;
		$length = unpack("N", $length);
	    }
	} else {
	    $type = ($tag >> 2) & 0x0F;
	    my $lentype = $tag & 0x03;
	    if ($lentype == 0) {
		$length = substr($data, $p, 1); $p += 1;
		$length = unpack("C", $length);
	    } elsif ($lentype == 1) {
		$length = substr($data, $p, 2); $p += 2;
		$length = unpack("n", $length);
	    } elsif ($lentype == 2) {
		$length = substr($data, $p, 4); $p += 4;
		$length = unpack("N", $length);
	    } elsif ($lentype == 3) {
		$length = undef;
	    }
	}

	if (defined $length) {
	    my $block;
	    unless (length($block = substr($data, $p, $length)) == $length) {
		warn "couldn't read $length bytes for packet body\n";
		return undef;
	    }
	    $pkt->{body} .= $block;
	    $p += $length;
	} else {
	    warn "unhandled variable length packet encountered ...\n";
	    return undef;
	}
    } while ($partial);

    $pkt->{tag} = $tag;
    $pkt->{type} = $type;

    return ($pkt, substr($data, $p));
}

my @n2s_types = ('Reserved',
		 'Public-Key Encrypted Session Key',
		 'Signature',
		 'Symmetric-Key Encrypted Session Key',
		 'One-Pass Signature',
		 'Secret Key',
		 'Public Key',
		 'Secret Subkey',
		 'Compressed Data',
		 'Symmetrically Encrypted Data',
		 'Marker Packet',
		 'Literal Data',
		 'Trust',
		 'User ID',
		 'Public Subkey');

my %s2n_types = map { ($n2s_types[$_], $_) } 0..$#n2s_types;

sub pgpapacketinfo {
    $_ = shift;

    if (/^-----BEGIN.*-----            \r?$ \n   # BEGIN something or other
         (?:^.+                        \r?$ \n)* # 0 or more headers
         ^                             \r?$ \n   # 1 empty line
         (
          (?:^[\+\/A-Za-z0-9]{64}      \r?$ \n)* # 0 or more lines of base64
          ^[\+\/A-Za-z0-9]{0,64}={0,2} \r?$ \n   # last line of base64
         )
         (^=[\+\/A-Za-z0-9]{4}         \r?$ \n)? # base64 crc line
         ^-----END.*-----              \r?$ \n   # END something or other
        /xm) {
	return pgppacketinfo(decode_base64($1));
    } else {
	return undef;
    }
}

sub pgppacketinfo {
    my $data = shift;
    my @packets;

    while (1) {
	(my $pkt, $data) = readpacket($data);
	last unless (defined $pkt);

	$pkt->{typename} = $n2s_types[$pkt->{type}] || 'Unknown';

	my $body = $pkt->{body};

	if ($pkt->{type} == 6) { # Public key
	    my $version = unpack("C", substr($body, 0, 1));
	    $body = substr($body, 1);
	    my ($time, $expire, $algo, $keyid32, $keyid64, $fingerprint);
	    if ($version == 3) {
		($time, $expire, $algo, $body) = unpack("NnCa*", $body);
		my ($mod, $exp);
		($mod, $body) = mpi($body);
		($exp, $body) = mpi($body);
		$keyid32 = substr($mod, length($mod) - 8);
		$fingerprint = join(' ', map { sprintf "%04.4X", $_ }
				    unpack("n*", MD5->hash($mod . $exp)));
	    } elsif ($version == 4) {
		($time, $algo, $body) = unpack("NCa*", $body);
		my $n =
		    # !! Note: we set the tag and len parts explicitly, 
		    #    as does gpg
		    SHA->hash(pack("C", 0x99). # tag (oldfmt,pubkey,lentype=16bit)
			      pack("n", 6 + length($body)). # len
			      pack("C", 4).         # V4
			      pack("N", $time).	# timestamp
			      pack("C", $algo).	# algorithm
			      $body); # remaining body is MPIs
		$fingerprint = join(' ', map { sprintf "%04.4X", $_ }
				    unpack("n*", $n));
		$keyid64 = join(' ', map { sprintf "%08.8X", $_ }
				unpack("N*", substr($n, -8)));
		$keyid32 = join(' ', map { sprintf "%08.8X", $_ }
				unpack("N*", substr($n, -4)));
	    } else {
		warn "encountered V$version Public Key packet version ?!\n";
		next;
	    }

	    $pkt->{pubkey}->{time} = $time;
	    $pkt->{pubkey}->{expire} = $expire;
	    $pkt->{pubkey}->{algo} = $algo;
	    $pkt->{pubkey}->{keyid32} = $keyid32;
	    $pkt->{pubkey}->{keyid64} = $keyid64;
	    $pkt->{pubkey}->{keyid} = (defined $keyid64) ? $keyid64 : $keyid32;
	    $pkt->{pubkey}->{fingerprint} = $fingerprint;
	} elsif ($pkt->{type} == 13) { # User ID
	    $pkt->{userid} = $body;
	} elsif ($pkt->{type} == 2) { # Signature
	    # these packets show up to verify the user id....
	    # but, whatever...
	}

	push(@packets, $pkt);
    }

    return @packets;
}

sub mpi {
    my $data = shift;
    my ($bits, $num);

    ($bits, $data) = unpack("na*", $data);
    my $length = int(($bits + 7) / 8);
    ($num, $data) = unpack("a$length" . "a*", $data);

    return (pack("na*", $bits, $num), $data);
}

1;

__END__

This is how fingerprints are done in gpg:

static MD_HANDLE
do_fingerprint_md( PKT_public_key *pk )
{
    MD_HANDLE md;
    unsigned n;
    unsigned nb[PUBKEY_MAX_NPKEY];
    unsigned nn[PUBKEY_MAX_NPKEY];
    byte *pp[PUBKEY_MAX_NPKEY];
    int i;
    int npkey = pubkey_get_npkey( pk->pubkey_algo );

    md = md_open( pk->version < 4 ? DIGEST_ALGO_RMD160 : DIGEST_ALGO_SHA1, 0);
    n = pk->version < 4 ? 8 : 6;
    for(i=0; i < npkey; i++ ) {
	nb[i] = mpi_get_nbits(pk->pkey[i]);
	pp[i] = mpi_get_buffer( pk->pkey[i], nn+i, NULL );
	n += 2 + nn[i];
    }

    md_putc( md, 0x99 );     /* ctb */
    md_putc( md, n >> 8 );   /* 2 byte length header */
    md_putc( md, n );
    if( pk->version < 4 )
	md_putc( md, 3 );
    else
	md_putc( md, 4 );

    {	u32 a = pk->timestamp;
	md_putc( md, a >> 24 );
	md_putc( md, a >> 16 );
	md_putc( md, a >>  8 );
	md_putc( md, a	     );
    }
    if( pk->version < 4 ) {
	u16 a;

	if( pk->expiredate )
	    a = (u16)((pk->expiredate - pk->timestamp) / 86400L);
	else
	    a = 0;
	md_putc( md, a >> 8 );
	md_putc( md, a	    );
    }
    md_putc( md, pk->pubkey_algo );
    for(i=0; i < npkey; i++ ) {
	md_putc( md, nb[i]>>8);
	md_putc( md, nb[i] );
	md_write( md, pp[i], nn[i] );
	m_free(pp[i]);
    }
    md_final( md );

    return md;
}
