#!/usr/bin/perl -w

my $mysql_server      = undef;
my $mysql_db          = 'pkdb';
my $mysql_user        = 'pk_admin';
my $mysql_pass        = undef;

use DBI;
use Mail::Address;
use PGPPacket;

use Data::Dumper;

my $dsn = "dbi:mysql:$mysql_db";
$dsn .= ":$mysql_server" if ((defined $mysql_server) and
			     ($mysql_server =~ /\S/));

my $db = DBI->connect($dsn, $mysql_user, $mysql_pass) or die $DBI::errstr;

my $st_addkey =
    $db->prepare('insert into db_ids2keys set ' .
		 'data = ?, ' .
		 'fingerprint = ? ');
my $st_addkey_id32 =
    $db->prepare('insert into short_key_id2db_ids set ' .
		 'key_id = conv(?, 16, 10), ' .
		 'db_id = ?');
my $st_addkey_id64 =
    $db->prepare('insert into long_key_id2db_ids set ' .
		 'lkey_id = conv(?, 16, 10), ' .
		 'hkey_id = conv(?, 16, 10), ' .
		 'db_id = ?');
my $st_addemail =
    $db->prepare('insert into emails2db_ids set ' .
		 'email = ?, ' .
		 'fullname = ?, ' .
		 'db_id = ?');

my $st_getkey =
    $db->prepare('select db_ids2keys.db_id ' .
		 'from db_ids2keys ' .
		 'left join emails2db_ids using (db_id) ' .
		 'left join short_key_id2db_ids using (db_id) ' .
		 'left join long_key_id2db_ids using (db_id) where ' .
		 'short_key_id2db_ids.key_id = conv(?, 16, 10) and ' .
		 '(long_key_id2db_ids.hkey_id = conv(?, 16, 10) or ' .
		 ' long_key_id2db_ids.hkey_id is NULL) and ' .
		 'email = ?');
my $st_getemail =
    $db->prepare('select email, fullname from emails2db_ids where ' .
		 'email = ? and ' .
		 'db_id = ?');

my $st_updatekey =
    $db->prepare('update db_ids2keys set ' .
		 'data = ? ' .
		 'where ' .
		 'db_id = ?');
my $st_updateemail =
    $db->prepare('update emails2db_ids set ' .
		 'new = 0, ' .
		 'fullname = ? ' .
		 'where ' .
		 'email = ? and ' .
		 'db_id = ?');

$/ = undef;

while (defined(my $block = <>)) {
    next unless (length $block);
    my $db_id;
    my %pgpkey;
    my @pgpkeys;

    foreach my $p (pgpapacketinfo($block)) {
	if (exists $p->{pubkey}) {
	    if (scalar %pgpkey > 0) {
		push(@pgpkeys, { %pgpkey });
	    }
	    %pgpkey = (%{$p->{pubkey}},
		       userid => []);
	} elsif (exists $p->{userid}) {
	    push(@{$pgpkey{userid}},
		 map { $_->phrase($_->comment) unless (defined $_->phrase);
		       $_; }
		 Mail::Address->parse($p->{userid}));
	}
    }
    push(@pgpkeys, { %pgpkey });

    foreach my $p (@pgpkeys) {
	my $lkeyid = "0x" . $p->{keyid32};
	$lkeyid =~ s/\s//g;
	my $hkeyid = "0x" . ($p->{keyid64} || 0);
	$hkeyid =~ s/\s.*//g;

	$db_id = undef;

	foreach my $a (@{$p->{userid}}) {
	    die $db->errstr
		unless ($st_getkey->execute($lkeyid,
					    $hkeyid,
					    $a->address));

	    if (my $row = $st_getkey->fetchrow_arrayref) {
		$db_id = $row->[0];
		$st_getkey->finish;
		last;
	    }
	}

	if ($db_id) {
	    my $mod;
	    print STDERR "Update key: $p->{keyid} (noop)\n";

	    foreach my $a (@{$p->{userid}}) {
		die $db->errstr
		    unless ($st_getemail->execute($a->address,
						  $db_id));

		if (my $row = $st_getemail->fetchrow_arrayref) {
		    if ($row->[1] ne $a->phrase) {
			printf(STDERR "Update email: %s <%s> -> %s <%s>\n",
			       $row->[1], $row->[0],
			       $a->phrase, $a->address);

			die $db->errstr
			    unless ($st_updateemail->execute($a->phrase,
							     $a->address,
							     $db_id));
			$mod++;
		    }
		    $st_getemail->finish;
		} else {
		    printf(STDERR "Insert email: %s <%s>\n",
			   $a->phrase, $a->address);

		    die $db->errstr
			unless ($st_addemail->execute($a->address,
						      $a->phrase,
						      $db_id));
		    $mod++;
		}
	    }

	    if ($mod) {
		die $db->errstr
		    unless ($st_updatekey->execute($block, $db_id));
	    }
	} else {
	    my $fp = $p->{fingerprint};
	    $fp =~ s/\s//g;

	    print STDERR "Insert key: $p->{keyid}\n";

	    die $db->errstr
		unless ($st_addkey->execute($block, $fp));
	    $db_id = $db->{mysql_insertid};

	    die $db_errstr
		unless ($st_addkey_id32->execute($lkeyid,
						 $db_id));

	    if (defined $p->{keyid64}) {
		die $db_errstr
		    unless ($st_addkey_id64->execute($lkeyid,
						     $hkeyid,
						     $db_id));
	    }

	    foreach my $a (@{$p->{userid}}) {
		printf(STDERR "Insert email: %s <%s>\n",
		       $a->phrase, $a->address);

		die $db->errstr
		    unless ($st_addemail->execute($a->address,
						  $a->phrase,
						  $db_id));
	    }
	}
    }
}

$db->disconnect;
