#!/usr/bin/perl -w

my $msg_subject       = "Mithril Key Server Synchronization";
my $msg_from          = 'kaj@dbd.com';
# my $msg_to            = 'db@securitier.com';
my $msg_to            = 'kaj@dbd.com';

my $mysql_server      = undef;
my $mysql_db          = 'pkdb';
my $mysql_user        = 'pk_admin';
my $mysql_pass        = undef;

my $site_name         = 'icryptogram.com';
my $server_name       = 'icryptogram.com';

my @domain_components = split(/\./, 'chainmailinc.com');

my $gpgd_secrets      = 'mithril-keyserver-sync-secrets';

use IO::Socket;

use DBI;

use Net::LDAP;
use Net::LDAP::Entry;
use Net::LDAP::LDIF;

use Mail::Send;

use FileHandle;

use IO::Scalar;

use Getopt::Long;

use PGPPacket;

GetOptions('mailer=s' => \ (my $mailer_cmd = 'sendmail'),
	   'raw!'     => \ (my $raw = 0));

my $dsn = "dbi:mysql:$mysql_db";
$dsn .= ":$mysql_server" if ((defined $mysql_server) and
			       ($mysql_server =~ /\S/));

my $db = DBI->connect($dsn, $mysql_user, $mysql_pass) or die $DBI::errstr;

my $ldif_buf;
tie *LDIF, IO::Scalar, \$ldif_buf;
my $out = Net::LDAP::LDIF->new(\*LDIF, "w");

my $st_getmods =
    $db->prepare('select ' .
		 'now() as new_sync, ' .
		 'db_ids2keys.db_id as db_id, ' .
		 'data as key_block, ' .
		 'fingerprint, ' .
		 'lpad(conv(key_id, 10, 16), 8, "0") as short_key_id, ' .
		 'concat(lpad(conv(hkey_id, 10, 16), 8, "0"), " ", ' .
		 '       lpad(conv(lkey_id, 10, 16), 8, "0")) ' .
		 '  as long_key_id, ' .
		 'mod, sync ' .
		 'from db_ids2keys ' .
		 'left join short_key_id2db_ids using (db_id) ' .
		 'left join long_key_id2db_ids using (db_id) ' .
		 'where ' .
		 'mod >= sync and mod < now()');

my $st_getemails =
    $db->prepare('select email from emails2db_ids where ' .
		 'db_id = ?');

my $st_marksynced =
    $db->prepare('update db_ids2keys set ' .
		 'mod = mod, ' .
		 'sync = ? ' .
		 'where ' .
		 'db_id = ?');

$st_getmods->execute;

while ($row = $st_getmods->fetchrow_hashref) {
    my $pgpobj = new Net::LDAP::Entry;
    my $method = $row->{sync} ? 'replace' : 'add';

    $pgpobj->$method(pgpKey         => $row->{key_block},
		     ou             => 'Key',
		     objectclass    => 'pgpObj',
		     status         => 'active',
		     pgpKeyId32     => $row->{short_key_id},
		     pgpFingerprint => $row->{fingerprint},
		     issuingServer  => encode_dn(undef, undef,
						 cn => $site_name,
						 ou => 'Server',
						 dc => [@domain_components]),
		     #issuedOn       => 0
		     );

    $pgpobj->$method(pgpKeyId64 => $row->{long_key_id})
	if ($row->{long_key_id});

    $st_getemails->execute($row->{db_id});
    $pgpobj->$method(mail => [map { $_->[0] }
			      @{ $st_getemails->fetchall_arrayref }]);

    # !! if mail changes, so does dn
    $pgpobj->dn(encode_dn($pgpobj, "mail+pgpKeyId32,ou",
			  dc => [@domain_components]));

    $st_marksynced->execute($row->{now}, $row->{db_id});

    $out->write($pgpobj);
}

$out->done;
close(LDIF);

exit unless ((length $ldif_buf) and ($ldif_buf =~ /\S/));

unless ($raw) {
    my $gpgd = new IO::Socket::UNIX(Peer => "/var/run/gpgd");

    open(GPGD_SECRETS, $gpgd_secrets) or
	die "$gpgd_secrets: $!\n";
    chomp (my $gpgd_user = <GPGD_SECRETS>);
    chomp (my $gpgd_pass = <GPGD_SECRETS>);
    close(GPGD_SECRETS);

    die "error talking to gpgd"
	unless ((split(/[\^\n]/, $gpgd->getline))[0] eq 'OK');

    $gpgd->print("LOGIN^$gpgd_user^$gpgd_pass\n");
    die "error talking to gpgd"
	unless ((split(/[\^\n]/, $gpgd->getline))[0] eq 'OK');

    $gpgd->print(sprintf("CRYPT^%d^SIGN^ENCRYPT^1\n", length $ldif_buf));
    die "error talking to gpgd"
	unless ((split(/[\^\n]/, $gpgd->getline))[0] eq 'OK');
    $gpgd->print("$msg_to\n$ldif_buf");
    die "error talking to gpgd"
	unless ($gpgd->getline =~ /^OK\^(\d+)\n$/);
    read($gpgd, $ldif_buf, $1);

    $gpgd->print("QUIT\n");
    close($gpgd);
}

$mail = Mail::Send->new(subject => $msg_subject,
			to      => $msg_to)->open($mailer_cmd);

$mail->print($ldif_buf);
close($mail);

$db->disconnect;

sub encode_dn_var {
    my $v = shift;

    $v =~ s/(,\+\"\\\<\>\;)/\\$1/g;
    $v =~ s/^(\s)/\\$1/;
    $v =~ s/(\s)$/\\$1/;

    return $v;
}

sub encode_dn {
    my $e = shift;
    my $fields = shift;
    my @others = @_;
    my $dn = '';

    if (defined $e and defined $fields) {
	my @fields = split(/([\s+,\+])/, $fields);

	while ((my $k = shift @fields) and
	       (my $op = @fields ? shift @fields : ',')) {
	    $op =~ s/\s*$//;
	    $op = ',' unless ($op =~ /\S/);
	    my $v = $e->get($k);
	    foreach $v (@{$v}) {
		$dn .= "$k=" . encode_dn_var($v) . $op . " ";
	    }
	}
    }

    while (@others >= 2) {
	my $k = shift @others;
	my $v = shift @others;

	my $op;
	$k =~ s/([,\+])?\s*$//;
	$op = $1 || ',';
	foreach $v (ref($v) eq 'ARRAY' ? @$v : $v) {
	    $dn .= "$k=" . encode_dn_var($v) . $op . " ";
	}
    }

    warn "odd number of key/value arguments passed to encode_dn: ",
	join(", ", @others)
	    if (scalar @others);

    $dn =~ s/[\+,] $//;
    return $dn;
}

