#ifndef SMTPCRYPT_H
#define SMTPCRYPT_H

#if 0
#  define SC_CONFIG_FILE "smtpcrypt.conf"
#else
#  define SC_CONFIG_FILE "/etc/mithril/smtpcrypt.conf"
#endif

#define SIGNING_PREFIX "insecure-"

/* this structure holds parameters for ESMTP MAIL and RCPT commands */
typedef struct _esmtp_address_parameter {
  char *name, *value;
  struct _esmtp_address_parameter *next;
} esmtp_address_parameter;

typedef enum {
    PGP_UNKNOWN,
    PGP_SIGN,
    PGP_ENCRYPT
} pgp_op_t;

typedef struct _esmtp_address {
  char *address;
  esmtp_address_parameter *params;
  pgp_op_t pgp_op;
  int has_key;
  struct _esmtp_address *next;
} esmtp_address;

typedef struct _smtpcrypt {
/* client/server IO */
  struct bio_st *client, *server;

/* SSL */
  struct ssl_ctx_st *ssl_ctx;
  struct ssl_st *client_ssl; /* no server ssl support yet ... */
  
/* logging, can be a file, syslog, stderr, etc */
  struct bio_st *log;

/* initial smtp server greeting received flag */
  enum {
    GREETING_NONE,
    GREETING_INITIAL,
    GREETING_HELO,
  } greeted;

/* commands pending completion (we support RFC2197 - PIPELINING extension) */
  struct _sc_command_pending *pending_commands, *pending_commands_tail;

/* HELO/EHLO hostname */
  char *helo_hostname;
  
/* identity */
  char *username;

/* crypt context */
  struct _MimeCrypt *mc;

/* envelope */
  struct {
    esmtp_address *from;        /* MAIL FROM: */
    esmtp_address *recipients;  /* RCPT TO:   */
  } env;

/* configuration file stuff */  
  struct {
    char *server_exec;

    char *server_addr;
    int server_port;

    char *log;

    char *greeting, *greeting_host;

    char *tls_enabled_s;
    enum {
      TLS_DISABLED,
      TLS_ENABLED,
      TLS_REQUIRED,
    } tls_enabled;
    
    char *tls_cert_file, *tls_key_file;

    int server_chat, client_chat;

    dbd_vec_t secure_prefixes, insecure_prefixes;

    int default_is_secure;

    int auth_requires_tls;
  } opt;
} smtpcrypt;

#define sc_log(sc, fmt, args...) BIO_printf(sc->log, fmt, ##args); BIO_write(sc->log, "\n", 1);

extern esmtp_address *esmtp_address_new_from_string(char *address);
extern void esmtp_address_destroy(esmtp_address *a);
extern void esmtp_address_parameter_destroy(esmtp_address_parameter *ap);

#endif /* SMTPCRYPT_H */
